# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.utils.module_loading import import_string
from django.conf import settings

models_AutoField = import_string(settings.DEFAULT_AUTO_FIELD)


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Basket',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(default='Open', max_length=128, verbose_name='Status', choices=[('Open', 'Open - currently active'), ('Merged', 'Merged - superceded by another basket'), ('Saved', 'Saved - for items to be purchased later'), ('Frozen', 'Frozen - the basket cannot be modified'), ('Submitted', 'Submitted - has been ordered at the checkout')])),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('date_merged', models.DateTimeField(blank=True, verbose_name='Date merged', null=True)),
                ('date_submitted', models.DateTimeField(blank=True, verbose_name='Date submitted', null=True)),
            ],
            options={
                'verbose_name_plural': 'Baskets',
                'verbose_name': 'Basket',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Line',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_reference', models.SlugField(max_length=128, verbose_name='Line Reference')),
                ('quantity', models.PositiveIntegerField(default=1, verbose_name='Quantity')),
                ('price_currency', models.CharField(default='GBP', max_length=12, verbose_name='Currency')),
                ('price_excl_tax', models.DecimalField(max_digits=12, decimal_places=2, verbose_name='Price excl. Tax', null=True)),
                ('price_incl_tax', models.DecimalField(max_digits=12, decimal_places=2, verbose_name='Price incl. Tax', null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
            ],
            options={
                'verbose_name_plural': 'Basket lines',
                'verbose_name': 'Basket line',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='LineAttribute',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=255, verbose_name='Value')),
                ('line', models.ForeignKey(verbose_name='Line', related_name='attributes', to='basket.Line', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name_plural': 'Line attributes',
                'verbose_name': 'Line attribute',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



from django import http
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.views.generic import ListView

from oscar.core.loading import get_model

ConditionalOffer = get_model("offer", "ConditionalOffer")
Range = get_model("offer", "Range")


class OfferListView(ListView):
    model = ConditionalOffer
    context_object_name = "offers"
    template_name = "oscar/offer/list.html"

    def get_queryset(self):
        """
        Return a queryset of active :py:class:`ConditionalOffer <oscar.apps.offer.abstract_models.AbstractConditionalOffer>`
        instances with an :py:attr:`offer_type <oscar.apps.offer.abstract_models.AbstractConditionalOffer.offer_type>`
        of :py:const:`ConditionalOffer.SITE <oscar.apps.offer.abstract_models.AbstractConditionalOffer.SITE>`.
        """
        return ConditionalOffer.active.filter(offer_type=ConditionalOffer.SITE)


class OfferDetailView(ListView):
    context_object_name = "products"
    template_name = "oscar/offer/detail.html"
    paginate_by = settings.OSCAR_OFFERS_PER_PAGE

    # pylint: disable=W0201
    def get(self, request, *args, **kwargs):
        try:
            self.offer = ConditionalOffer.active.select_related().get(
                slug=self.kwargs["slug"]
            )
        except ConditionalOffer.DoesNotExist:
            raise http.Http404
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["offer"] = self.offer
        ctx["upsell_message"] = self.offer.get_upsell_message(self.request.basket)
        return ctx

    def get_queryset(self):
        """
        Return a queryset of all :py:class:`Product <oscar.apps.catalogue.abstract_models.AbstractProduct>`
        instances related to the :py:class:`ConditionalOffer <oscar.apps.offer.abstract_models.AbstractConditionalOffer>`.
        """
        return self.offer.products()


class RangeDetailView(ListView):
    template_name = "oscar/offer/range.html"
    context_object_name = "products"
    paginate_by = settings.OSCAR_PRODUCTS_PER_PAGE

    # pylint: disable=W0201
    def dispatch(self, request, *args, **kwargs):
        self.range = get_object_or_404(Range, slug=kwargs["slug"], is_public=True)
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        """
        Return a queryset of all :py:class:`Product <oscar.apps.catalogue.abstract_models.AbstractProduct>`
        instances related to the :py:class:`Range <oscar.apps.offer.abstract_models.AbstractRange>`.
        """
        products = self.range.all_products().browsable()
        return products.order_by("rangeproduct__display_order")

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["range"] = self.range
        return ctx



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0006_auto_20150807_1725'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='attributeoption',
            unique_together=set([('group', 'option')]),
        ),
    ]



import logging
import os
import shutil

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Copy Oscar's statics into local project so they can be used as a base for
    styling a new site.
    """

    help = "Copy Oscar's static files"

    def add_arguments(self, parser):
        parser.add_argument(
            "target_path",
            nargs="?",
            default="static",
            help="Directory path (relative to current working "
            "directory, or absolute) in local project, to "
            "copy files to. This directory should be "
            "included in STATICFILES_DIRS, in order for "
            "your local overrides to be picked up.",
        )

    def handle(self, *args, **options):
        # Determine where to copy to
        folder = options["target_path"]
        if not folder.startswith("/"):
            destination = os.path.join(os.getcwd(), folder)
        else:
            destination = folder
        if os.path.exists(destination):
            raise CommandError("The folder %s already exists - aborting!" % destination)

        source = os.path.realpath(
            os.path.join(os.path.dirname(__file__), "../../static")
        )
        self.stdout.write("Copying Oscar's static files to %s" % destination)
        shutil.copytree(source, destination)

        # Check if this new folder is in STATICFILES_DIRS
        if destination not in settings.STATICFILES_DIRS:
            self.stdout.write(
                "You need to add %s to STATICFILES_DIRS in order for your local overrides to be picked "
                "up" % destination
            )



from django.contrib import admin
from treebeard.admin import TreeAdmin
from treebeard.forms import movenodeform_factory

from oscar.core.loading import get_model

AttributeOption = get_model("catalogue", "AttributeOption")
AttributeOptionGroup = get_model("catalogue", "AttributeOptionGroup")
Category = get_model("catalogue", "Category")
Option = get_model("catalogue", "Option")
Product = get_model("catalogue", "Product")
ProductAttribute = get_model("catalogue", "ProductAttribute")
ProductAttributeValue = get_model("catalogue", "ProductAttributeValue")
ProductCategory = get_model("catalogue", "ProductCategory")
ProductClass = get_model("catalogue", "ProductClass")
ProductImage = get_model("catalogue", "ProductImage")
ProductRecommendation = get_model("catalogue", "ProductRecommendation")


class AttributeInline(admin.TabularInline):
    model = ProductAttributeValue


class ProductRecommendationInline(admin.TabularInline):
    model = ProductRecommendation
    fk_name = "primary"
    raw_id_fields = ["primary", "recommendation"]


class CategoryInline(admin.TabularInline):
    model = ProductCategory
    extra = 1


class ProductAttributeInline(admin.TabularInline):
    model = ProductAttribute
    extra = 2


class ProductClassAdmin(admin.ModelAdmin):
    list_display = ("name", "requires_shipping", "track_stock")
    inlines = [ProductAttributeInline]


class ProductAdmin(admin.ModelAdmin):
    date_hierarchy = "date_created"
    list_display = (
        "get_title",
        "upc",
        "get_product_class",
        "structure",
        "attribute_summary",
        "date_created",
    )
    list_filter = ["structure", "is_discountable"]
    raw_id_fields = ["parent"]
    inlines = [AttributeInline, CategoryInline, ProductRecommendationInline]
    prepopulated_fields = {"slug": ("title",)}
    search_fields = ["upc", "title"]

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related("product_class", "parent").prefetch_related(
            "attribute_values", "attribute_values__attribute"
        )


class ProductAttributeAdmin(admin.ModelAdmin):
    list_display = ("name", "code", "product_class", "type")
    prepopulated_fields = {"code": ("name",)}


class OptionAdmin(admin.ModelAdmin):
    pass


class ProductAttributeValueAdmin(admin.ModelAdmin):
    list_display = ("product", "attribute", "value")


class AttributeOptionInline(admin.TabularInline):
    model = AttributeOption


class AttributeOptionGroupAdmin(admin.ModelAdmin):
    list_display = ("name", "option_summary")
    inlines = [
        AttributeOptionInline,
    ]


class CategoryAdmin(TreeAdmin):
    form = movenodeform_factory(Category)
    list_display = ("name", "slug")


admin.site.register(ProductClass, ProductClassAdmin)
admin.site.register(Product, ProductAdmin)
admin.site.register(ProductAttribute, ProductAttributeAdmin)
admin.site.register(ProductAttributeValue, ProductAttributeValueAdmin)
admin.site.register(AttributeOptionGroup, AttributeOptionGroupAdmin)
admin.site.register(Option, OptionAdmin)
admin.site.register(ProductImage)
admin.site.register(Category, CategoryAdmin)
admin.site.register(ProductCategory)



# Generated by Django 3.0.8 on 2020-08-01 07:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('voucher', '0007_auto_20181115_1953'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='voucher',
            options={'get_latest_by': 'date_created', 'ordering': ['-date_created'], 'verbose_name': 'Voucher', 'verbose_name_plural': 'Vouchers'},
        ),
        migrations.AlterModelOptions(
            name='voucherapplication',
            options={'ordering': ['-date_created'], 'verbose_name': 'Voucher Application', 'verbose_name_plural': 'Voucher Applications'},
        ),
        migrations.AlterModelOptions(
            name='voucherset',
            options={'get_latest_by': 'date_created', 'ordering': ['-date_created'], 'verbose_name': 'VoucherSet', 'verbose_name_plural': 'VoucherSets'},
        ),
        migrations.AlterField(
            model_name='voucher',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name='voucherapplication',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name='voucherset',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
    ]



import re

from django import forms
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from oscar.core.loading import get_model

Product = get_model("catalogue", "Product")
Range = get_model("offer", "Range")
RangeProductFileUpload = get_model("offer", "RangeProductFileUpload")

UPC_SET_REGEX = re.compile(r"[^,\s]+")


class RangeForm(forms.ModelForm):
    class Meta:
        model = Range
        fields = [
            "name",
            "description",
            "is_public",
            "includes_all_products",
            "included_categories",
            "excluded_categories",
        ]


# pylint: disable=attribute-defined-outside-init
class RangeProductForm(forms.Form):
    query = forms.CharField(
        max_length=1024,
        label=_("Product SKUs or UPCs"),
        widget=forms.Textarea,
        required=False,
        help_text=_("You can paste in a selection of SKUs or UPCs"),
    )
    file_upload = forms.FileField(
        label=_("File of SKUs or UPCs"),
        required=False,
        max_length=255,
        help_text=_("Either comma-separated, or one identifier per line"),
    )
    upload_type = forms.CharField(widget=forms.HiddenInput(), required=False)

    def __init__(self, product_range, *args, **kwargs):
        self.product_range = product_range
        super().__init__(*args, **kwargs)

    def clean_query_with_upload_type(self, raw, upload_type):
        # Check that the search matches some products
        ids = set(UPC_SET_REGEX.findall(raw))
        # switch for included or excluded products
        if upload_type == RangeProductFileUpload.EXCLUDED_PRODUCTS_TYPE:
            products = self.product_range.excluded_products.all()
            action = _("excluded from this range")
        else:
            products = self.product_range.all_products()
            action = _("added to this range")
        existing_skus = set(
            products.values_list("stockrecords__partner_sku", flat=True)
        )
        existing_upcs = set(products.values_list("upc", flat=True))
        existing_ids = existing_skus.union(existing_upcs)
        new_ids = ids - existing_ids
        if len(new_ids) == 0:
            self.add_error(
                "query",
                _(
                    "The products with SKUs or UPCs matching %(skus)s have "
                    "already been %(action)s"
                )
                % {"skus": ", ".join(ids), "action": action},
            )
        else:
            self.products = Product._default_manager.filter(
                Q(stockrecords__partner_sku__in=new_ids) | Q(upc__in=new_ids)
            )
            if len(self.products) == 0:
                self.add_error(
                    "query",
                    _("No products exist with a SKU or UPC matching %s")
                    % ", ".join(ids),
                )
            found_skus = set(
                self.products.values_list("stockrecords__partner_sku", flat=True)
            )
            found_upcs = set(self.products.values_list("upc", flat=True))
            found_ids = found_skus.union(found_upcs)
            self.missing_skus = new_ids - found_ids
            self.duplicate_skus = existing_ids.intersection(ids)

    def clean(self):
        clean_data = super().clean()
        if not clean_data.get("query") and not clean_data.get("file_upload"):
            raise forms.ValidationError(
                _("You must submit either a list of SKU/UPCs or a file")
            )
        raw = clean_data["query"]
        if raw:
            self.clean_query_with_upload_type(raw, clean_data["upload_type"])
        return clean_data

    def get_products(self):
        return self.products if hasattr(self, "products") else []

    def get_missing_skus(self):
        return self.missing_skus

    def get_duplicate_skus(self):
        return self.duplicate_skus


# pylint: disable=attribute-defined-outside-init
class RangeExcludedProductForm(RangeProductForm):
    """
    Form to add products in range.excluded_products
    """

    def clean_query(self):
        raw = self.cleaned_data["query"]
        if not raw:
            return raw

        # Check that the search matches some products
        ids = set(UPC_SET_REGEX.findall(raw))
        products = self.product_range.excluded_products.all()
        existing_skus = set(
            products.values_list("stockrecords__partner_sku", flat=True)
        )
        existing_upcs = set(products.values_list("upc", flat=True))
        existing_ids = existing_skus.union(existing_upcs)
        new_ids = ids - existing_ids

        if len(new_ids) == 0:
            raise forms.ValidationError(
                _(
                    "The products with SKUs or UPCs matching %s are already in"
                    " this range"
                )
                % (", ".join(ids))
            )

        self.products = Product._default_manager.filter(
            Q(stockrecords__partner_sku__in=new_ids) | Q(upc__in=new_ids)
        )
        if len(self.products) == 0:
            raise forms.ValidationError(
                _("No products exist with a SKU or UPC matching %s") % ", ".join(ids)
            )

        found_skus = set(
            self.products.values_list("stockrecords__partner_sku", flat=True)
        )
        found_upcs = set(self.products.values_list("upc", flat=True))
        found_ids = found_skus.union(found_upcs)
        self.missing_skus = new_ids - found_ids
        self.duplicate_skus = existing_ids.intersection(ids)

        return raw



# coding=utf-8
import datetime
import random
from decimal import Decimal as D

from django.conf import settings
from django.utils import timezone

from oscar.core.loading import get_class, get_model
from oscar.test.factories.address import *
from oscar.test.factories.basket import *
from oscar.test.factories.catalogue import *
from oscar.test.factories.contrib import *
from oscar.test.factories.customer import *
from oscar.test.factories.models import *
from oscar.test.factories.offer import *
from oscar.test.factories.order import *
from oscar.test.factories.partner import *
from oscar.test.factories.payment import *
from oscar.test.factories.voucher import *
from oscar.test.factories.wishlists import *

Basket = get_model("basket", "Basket")
Free = get_class("shipping.methods", "Free")
Voucher = get_model("voucher", "Voucher")
OrderCreator = get_class("order.utils", "OrderCreator")
OrderTotalCalculator = get_class("checkout.calculators", "OrderTotalCalculator")
SurchargeApplicator = get_class("checkout.applicator", "SurchargeApplicator")
Partner = get_model("partner", "Partner")
StockRecord = get_model("partner", "StockRecord")
PurchaseInfo = get_class("partner.strategy", "PurchaseInfo")
Default = get_class("partner.strategy", "Default")
StockRequired = get_class("partner.availability", "StockRequired")
FixedPrice = get_class("partner.prices", "FixedPrice")

Product = get_model("catalogue", "Product")
ProductClass = get_model("catalogue", "ProductClass")
ProductAttribute = get_model("catalogue", "ProductAttribute")
ProductAttributeValue = get_model("catalogue", "ProductAttributeValue")
ProductImage = get_model("catalogue", "ProductImage")

WeightBand = get_model("shipping", "WeightBand")
WeightBased = get_model("shipping", "WeightBased")

Range = get_model("offer", "Range")
Condition = get_model("offer", "Condition")
Benefit = get_model("offer", "Benefit")
ConditionalOffer = get_model("offer", "ConditionalOffer")


def create_stockrecord(
    product=None,
    price=None,
    partner_sku=None,
    num_in_stock=None,
    partner_name=None,
    currency=settings.OSCAR_DEFAULT_CURRENCY,
    partner_users=None,
):
    if product is None:
        product = create_product()
    partner, __ = Partner.objects.get_or_create(name=partner_name or "")
    if partner_users:
        for user in partner_users:
            partner.users.add(user)
    if price is None:
        price = D("9.99")
    if partner_sku is None:
        partner_sku = "sku_%d_%d" % (product.id, random.randint(0, 10000))
    return product.stockrecords.create(
        partner=partner,
        partner_sku=partner_sku,
        price_currency=currency,
        price=price,
        num_in_stock=num_in_stock,
    )


def create_purchase_info(record):
    return PurchaseInfo(
        price=FixedPrice(
            record.price_currency, record.price, D("0.00")  # Default to no tax
        ),
        availability=StockRequired(record.net_stock_level),
        stockrecord=record,
    )


def create_product(
    upc=None,
    title="Dùｍϻϒ title",
    product_class="Dùｍϻϒ item class",
    partner_name=None,
    partner_sku=None,
    price=None,
    num_in_stock=None,
    attributes=None,
    partner_users=None,
    **kwargs,
):
    """
    Helper method for creating products that are used in tests.
    """
    product_class, __ = ProductClass._default_manager.get_or_create(name=product_class)
    product = product_class.products.model(
        product_class=product_class, title=title, upc=upc, **kwargs
    )
    if kwargs.get("parent") and "structure" not in kwargs:
        product.structure = "child"
    if attributes:
        for code, value in attributes.items():
            # Ensure product attribute exists
            product_class.attributes.get_or_create(name=code, code=code)
            setattr(product.attr, code, value)
    product.save()

    # Shortcut for creating stockrecord
    stockrecord_fields = [price, partner_sku, partner_name, num_in_stock, partner_users]
    if any([field is not None for field in stockrecord_fields]):
        create_stockrecord(
            product,
            price=price,
            num_in_stock=num_in_stock,
            partner_users=partner_users,
            partner_sku=partner_sku,
            partner_name=partner_name,
        )
    return product


def create_product_image(
    product=None,
    original=None,
    caption="Dummy Caption",
    display_order=None,
):
    if not product:
        product = create_product()
    if not display_order:
        if not product.images.all():
            display_order = 0
        else:
            display_order = max([i.display_order for i in product.images.all()]) + 1

    kwargs = {
        "product_id": product.id,
        "original": original,
        "display_order": display_order,
        "caption": caption,
    }

    return ProductImage.objects.create(**kwargs)


def create_basket(empty=False):
    basket = Basket.objects.create()
    basket.strategy = Default()
    if not empty:
        product = create_product()
        create_stockrecord(product, num_in_stock=2)
        basket.add_product(product)
    return basket


def create_order(
    number=None,
    basket=None,
    user=None,
    shipping_address=None,
    shipping_method=None,
    billing_address=None,
    total=None,
    **kwargs,
):
    """
    Helper method for creating an order for testing
    """
    if not basket:
        basket = Basket.objects.create()
        basket.strategy = Default()
        product = create_product()
        create_stockrecord(product, num_in_stock=10, price=D("10.00"))
        basket.add_product(product)
    if not basket.id:
        basket.save()
    if shipping_method is None:
        shipping_method = Free()
    shipping_charge = shipping_method.calculate(basket)
    surcharges = SurchargeApplicator().get_applicable_surcharges(basket)
    if total is None:
        total = OrderTotalCalculator().calculate(basket, shipping_charge, surcharges)
    kwargs["surcharges"] = surcharges
    order = OrderCreator().place_order(
        order_number=number,
        user=user,
        basket=basket,
        shipping_address=shipping_address,
        shipping_method=shipping_method,
        shipping_charge=shipping_charge,
        billing_address=billing_address,
        total=total,
        **kwargs,
    )
    basket.set_as_submitted()
    return order


def create_offer(
    name="Dùｍϻϒ offer",
    offer_type="Site",
    max_basket_applications=None,
    product_range=None,
    condition=None,
    benefit=None,
    priority=0,
    status=None,
    start=None,
    end=None,
):
    """
    Helper method for creating an offer
    """
    if product_range is None:
        product_range, __ = Range.objects.get_or_create(
            name="All products räñgë", includes_all_products=True
        )
    if condition is None:
        condition, __ = Condition.objects.get_or_create(
            range=product_range, type=Condition.COUNT, value=1
        )
    if benefit is None:
        benefit, __ = Benefit.objects.get_or_create(
            range=product_range, type=Benefit.PERCENTAGE, value=20
        )
    if status is None:
        status = ConditionalOffer.OPEN

    # Create start and end date so offer is active
    now = timezone.now()  # pylint:disable=W0621
    if start is None:
        start = now - datetime.timedelta(days=1)
    if end is None:
        end = now + datetime.timedelta(days=30)

    return ConditionalOffer.objects.create(
        name=name,
        start_datetime=start,
        end_datetime=end,
        status=status,
        offer_type=offer_type,
        condition=condition,
        benefit=benefit,
        max_basket_applications=max_basket_applications,
        priority=priority,
    )


def create_voucher(**kwargs):
    """
    Helper method for creating a voucher
    """
    defaults = {
        "name": "Dùｍϻϒ voucher",
        "code": "test",
        "start_datetime": timezone.now(),
        "end_datetime": timezone.now() + datetime.timedelta(days=12),
    }
    defaults.update(kwargs)
    voucher = VoucherFactory(**defaults)
    voucher.offers.add(create_offer(offer_type="Voucher"))
    return voucher


def create_shipping_weight_based(default_weight=D(1)):
    return WeightBased.objects.create(default_weight=default_weight)


def create_shipping_weight_band(upper_limit, charge, weight_based=None):
    if not weight_based:
        weight_based = create_shipping_weight_based()
    return WeightBand.objects.create(
        method=weight_based, upper_limit=upper_limit, charge=charge
    )



from collections import defaultdict

from django import forms
from django.conf import settings
from django.forms.widgets import Input
from django.utils.translation import gettext_lazy as _

from haystack.forms import FacetedSearchForm

from oscar.core.loading import get_class

is_solr_supported = get_class("search.features", "is_solr_supported")


class SearchInput(Input):
    """
    Defining a search type widget

    This is an HTML5 thing and works nicely with Safari, other browsers default
    back to using the default "text" type
    """

    input_type = "search"


# Build a dict of valid queries
VALID_FACET_QUERIES = defaultdict(list)
for facet in settings.OSCAR_SEARCH_FACETS["queries"].values():
    name = facet["field"]
    queries = [t[1] for t in facet["queries"]]
    VALID_FACET_QUERIES[name].extend(queries)


class SearchForm(FacetedSearchForm):
    """
    In Haystack, the search form is used for interpreting
    and sub-filtering the SQS.
    """

    # Use a tabindex of 1 so that users can hit tab on any page and it will
    # focus on the search widget.
    q = forms.CharField(
        required=False,
        label=_("Search"),
        widget=SearchInput(
            {"placeholder": _("Search"), "tabindex": "1", "class": "form-control"}
        ),
    )

    # Search
    RELEVANCY = "relevancy"
    TOP_RATED = "rating"
    NEWEST = "newest"
    PRICE_HIGH_TO_LOW = "price-desc"
    PRICE_LOW_TO_HIGH = "price-asc"
    TITLE_A_TO_Z = "title-asc"
    TITLE_Z_TO_A = "title-desc"

    SORT_BY_CHOICES = [
        (RELEVANCY, _("Relevancy")),
        (TOP_RATED, _("Customer rating")),
        (NEWEST, _("Newest")),
        (PRICE_HIGH_TO_LOW, _("Price high to low")),
        (PRICE_LOW_TO_HIGH, _("Price low to high")),
        (TITLE_A_TO_Z, _("Title A to Z")),
        (TITLE_Z_TO_A, _("Title Z to A")),
    ]

    # Map query params to sorting fields.  Note relevancy isn't included here
    # as we assume results are returned in relevancy order in the absence of an
    # explicit sort field being passed to the search backend.
    SORT_BY_MAP = {
        TOP_RATED: "-rating",
        NEWEST: "-date_created",
        PRICE_HIGH_TO_LOW: "-price",
        PRICE_LOW_TO_HIGH: "price",
        TITLE_A_TO_Z: "title_s",
        TITLE_Z_TO_A: "-title_s",
    }
    # Non Solr backends don't support dynamic fields so we just sort on title
    if not is_solr_supported():
        SORT_BY_MAP[TITLE_A_TO_Z] = "title_exact"
        SORT_BY_MAP[TITLE_Z_TO_A] = "-title_exact"

    sort_by = forms.ChoiceField(
        label=_("Sort by"),
        choices=SORT_BY_CHOICES,
        widget=forms.Select(),
        required=False,
    )

    @property
    def selected_multi_facets(self):
        """
        Validate and return the selected facets
        """
        # Process selected facets into a dict(field->[*values]) to handle
        # multi-faceting
        selected_multi_facets = defaultdict(list)

        for facet_kv in self.selected_facets:
            if ":" not in facet_kv:
                continue
            field_name, value = facet_kv.split(":", 1)

            # Validate query facets as they as passed unescaped to Solr
            if field_name in VALID_FACET_QUERIES:
                if value not in VALID_FACET_QUERIES[field_name]:
                    # Invalid query value
                    continue

            selected_multi_facets[field_name].append(value)

        return selected_multi_facets

    def search(self):
        # We replace the 'search' method from FacetedSearchForm, so that we can
        # handle range queries
        # Note, we call super on a parent class as the default faceted view
        # escapes everything (which doesn't work for price range queries)
        sqs = super(FacetedSearchForm, self).search()

        # We need to process each facet to ensure that the field name and the
        # value are quoted correctly and separately:
        for field, values in self.selected_multi_facets.items():
            if not values:
                continue
            if field in VALID_FACET_QUERIES:
                # Query facet - don't wrap value in speech marks and don't
                # clean value. Query values should have been validated by this
                # point and so we don't need to escape them.
                sqs = sqs.narrow("%s:(%s)" % (field, " OR ".join(values)))
            else:
                # Field facet - clean and quote the values
                clean_values = ['"%s"' % sqs.query.clean(val) for val in values]
                sqs = sqs.narrow("%s:(%s)" % (field, " OR ".join(clean_values)))

        if self.is_valid() and "sort_by" in self.cleaned_data:
            sort_field = self.SORT_BY_MAP.get(self.cleaned_data["sort_by"], None)
            if sort_field:
                sqs = sqs.order_by(sort_field)

        return sqs


class BrowseCategoryForm(SearchForm):
    """
    Variant of SearchForm that returns all products (instead of none) if no
    query is specified.
    """

    def no_query_found(self):
        """
        Return Queryset of all the results.
        """
        return self.searchqueryset


class CategoryForm(BrowseCategoryForm):
    def __init__(self, categories, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.categories = categories

    def no_query_found(self):
        """
        Return Queryset of all the results.
        """
        sqs = super().no_query_found()

        category_ids = list(self.categories.values_list("pk", flat=True))
        return sqs.filter(category__in=category_ids)



# pylint: disable=wildcard-import, unused-wildcard-import
from oscar.apps.address.abstract_models import (
    AbstractBillingAddress,
    AbstractShippingAddress,
)
from oscar.apps.order.abstract_models import *
from oscar.core.loading import is_model_registered

__all__ = ["PaymentEventQuantity", "ShippingEventQuantity"]


if not is_model_registered("order", "Order"):

    class Order(AbstractOrder):
        pass

    __all__.append("Order")


if not is_model_registered("order", "OrderNote"):

    class OrderNote(AbstractOrderNote):
        pass

    __all__.append("OrderNote")


if not is_model_registered("order", "OrderStatusChange"):

    class OrderStatusChange(AbstractOrderStatusChange):
        pass

    __all__.append("OrderStatusChange")


if not is_model_registered("order", "CommunicationEvent"):

    class CommunicationEvent(AbstractCommunicationEvent):
        pass

    __all__.append("CommunicationEvent")


if not is_model_registered("order", "ShippingAddress"):

    class ShippingAddress(AbstractShippingAddress):
        pass

    __all__.append("ShippingAddress")


if not is_model_registered("order", "BillingAddress"):

    class BillingAddress(AbstractBillingAddress):
        pass

    __all__.append("BillingAddress")


if not is_model_registered("order", "Line"):

    class Line(AbstractLine):
        pass

    __all__.append("Line")


if not is_model_registered("order", "LinePrice"):

    class LinePrice(AbstractLinePrice):
        pass

    __all__.append("LinePrice")


if not is_model_registered("order", "LineAttribute"):

    class LineAttribute(AbstractLineAttribute):
        pass

    __all__.append("LineAttribute")


if not is_model_registered("order", "ShippingEvent"):

    class ShippingEvent(AbstractShippingEvent):
        pass

    __all__.append("ShippingEvent")


if not is_model_registered("order", "ShippingEventType"):

    class ShippingEventType(AbstractShippingEventType):
        pass

    __all__.append("ShippingEventType")


if not is_model_registered("order", "PaymentEvent"):

    class PaymentEvent(AbstractPaymentEvent):
        pass

    __all__.append("PaymentEvent")


if not is_model_registered("order", "PaymentEventType"):

    class PaymentEventType(AbstractPaymentEventType):
        pass

    __all__.append("PaymentEventType")


if not is_model_registered("order", "OrderDiscount"):

    class OrderDiscount(AbstractOrderDiscount):
        pass

    __all__.append("OrderDiscount")


if not is_model_registered("order", "OrderLineDiscount"):

    class OrderLineDiscount(AbstractOrderLineDiscount):
        pass

    __all__.append("OrderLineDiscount")


if not is_model_registered("order", "Surcharge"):

    class Surcharge(AbstractSurcharge):
        pass

    __all__.append("Surcharge")



from django.contrib import admin

from oscar.core.loading import get_model

Partner = get_model("partner", "Partner")
StockRecord = get_model("partner", "StockRecord")


class StockRecordAdmin(admin.ModelAdmin):
    list_display = ("product", "partner", "partner_sku", "price", "num_in_stock")
    list_filter = ("partner",)


admin.site.register(Partner)
admin.site.register(StockRecord, StockRecordAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-04-13 17:53
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('customer', '0003_update_email_length'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='email',
            field=models.EmailField(blank=True, max_length=254, null=True, verbose_name='Email Address'),
        ),
        migrations.AlterField(
            model_name='email',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='emails', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



from decimal import Decimal

from django.core import exceptions
from django.db import models
from django.db.models import Sum
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from oscar.core.compat import AUTH_USER_MODEL
from oscar.core.decorators import deprecated


class AbstractVoucherSet(models.Model):
    """A collection of vouchers (potentially auto-generated)

    a VoucherSet is a group of voucher that are generated
    automatically.

    - count: the number of vouchers in the set. If this is kept at
    zero, vouchers are created when and as needed.

    - code_length: the length of the voucher code. Codes are by default created
    with groups of 4 characters: XXXX-XXXX-XXXX. The dashes (-) do not count for
    the code_length.

    - :py:attr:`.start_datetime` and :py:attr:`.end_datetime` together define the validity
      range for all vouchers in the set.
    """

    name = models.CharField(verbose_name=_("Name"), max_length=100, unique=True)
    count = models.PositiveIntegerField(verbose_name=_("Number of vouchers"))
    code_length = models.IntegerField(verbose_name=_("Length of Code"), default=12)
    description = models.TextField(verbose_name=_("Description"))
    date_created = models.DateTimeField(auto_now_add=True, db_index=True)
    start_datetime = models.DateTimeField(_("Start datetime"))
    end_datetime = models.DateTimeField(_("End datetime"))

    class Meta:
        abstract = True
        app_label = "voucher"
        get_latest_by = "date_created"
        ordering = ["-date_created"]
        verbose_name = _("VoucherSet")
        verbose_name_plural = _("VoucherSets")

    def __str__(self):
        return self.name

    def clean(self):
        if (
            self.start_datetime
            and self.end_datetime
            and (self.start_datetime > self.end_datetime)
        ):
            raise exceptions.ValidationError(
                _("End date should be later than start date")
            )

    def update_count(self):
        vouchers_count = self.vouchers.count()
        if self.count != vouchers_count:
            self.count = vouchers_count
            self.save()

    def is_active(self, test_datetime=None):
        """Test whether this voucher set is currently active."""
        test_datetime = test_datetime or timezone.now()
        return self.start_datetime <= test_datetime <= self.end_datetime

    @property
    def num_basket_additions(self):
        value = self.vouchers.aggregate(result=Sum("num_basket_additions"))
        return value["result"]

    @property
    def num_orders(self):
        value = self.vouchers.aggregate(result=Sum("num_orders"))
        return value["result"]

    @property
    def total_discount(self):
        value = self.vouchers.aggregate(result=Sum("total_discount"))
        return value["result"]


class AbstractVoucher(models.Model):
    """
    A voucher.  This is simply a link to a collection of offers.

    Note that there are three possible "usage" modes:
    (a) Single use
    (b) Multi-use
    (c) Once per customer

    Oscar enforces those modes by creating VoucherApplication
    instances when a voucher is used for an order.
    """

    name = models.CharField(
        _("Name"),
        max_length=128,
        unique=True,
        help_text=_(
            "This will be shown in the checkout"
            " and basket once the voucher is"
            " entered"
        ),
    )
    code = models.CharField(
        _("Code"),
        max_length=128,
        db_index=True,
        unique=True,
        help_text=_("Case insensitive / No spaces allowed"),
    )
    offers = models.ManyToManyField(
        "offer.ConditionalOffer",
        related_name="vouchers",
        verbose_name=_("Offers"),
        limit_choices_to={"offer_type": "Voucher"},
    )

    SINGLE_USE, MULTI_USE, ONCE_PER_CUSTOMER = (
        "Single use",
        "Multi-use",
        "Once per customer",
    )
    USAGE_CHOICES = (
        (SINGLE_USE, _("Can be used once by one customer")),
        (MULTI_USE, _("Can be used multiple times by multiple customers")),
        (ONCE_PER_CUSTOMER, _("Can only be used once per customer")),
    )
    usage = models.CharField(
        _("Usage"), max_length=128, choices=USAGE_CHOICES, default=MULTI_USE
    )

    start_datetime = models.DateTimeField(_("Start datetime"), db_index=True)
    end_datetime = models.DateTimeField(_("End datetime"), db_index=True)

    # Reporting information. Not used to enforce any consumption limits.
    num_basket_additions = models.PositiveIntegerField(
        _("Times added to basket"), default=0
    )
    num_orders = models.PositiveIntegerField(_("Times on orders"), default=0)
    total_discount = models.DecimalField(
        _("Total discount"), decimal_places=2, max_digits=12, default=Decimal("0.00")
    )

    voucher_set = models.ForeignKey(
        "voucher.VoucherSet",
        null=True,
        blank=True,
        related_name="vouchers",
        on_delete=models.CASCADE,
    )

    date_created = models.DateTimeField(auto_now_add=True, db_index=True)

    class Meta:
        abstract = True
        app_label = "voucher"
        ordering = ["-date_created"]
        get_latest_by = "date_created"
        verbose_name = _("Voucher")
        verbose_name_plural = _("Vouchers")

    def __str__(self):
        return self.name

    def clean(self):
        if (
            self.start_datetime
            and self.end_datetime
            and (self.start_datetime > self.end_datetime)
        ):
            raise exceptions.ValidationError(
                _("End date should be later than start date")
            )

    def save(self, *args, **kwargs):
        self.code = self.code.upper()
        super().save(*args, **kwargs)

    def is_active(self, test_datetime=None):
        """
        Test whether this voucher is currently active.
        """
        test_datetime = test_datetime or timezone.now()
        return self.start_datetime <= test_datetime <= self.end_datetime

    def is_expired(self):
        """
        Test whether this voucher has passed its expiration date
        """
        now = timezone.now()
        return self.end_datetime < now

    def is_available_to_user(self, user=None):
        """
        Test whether this voucher is available to the passed user.

        Returns a tuple of a boolean for whether it is successful, and a
        availability message.
        """
        is_available, message = False, ""
        if self.usage == self.SINGLE_USE:
            is_available = not self.applications.exists()
            if not is_available:
                message = _("This voucher has already been used")
        elif self.usage == self.MULTI_USE:
            is_available = True
        elif self.usage == self.ONCE_PER_CUSTOMER:
            if not user.is_authenticated:
                is_available = False
                message = _("This voucher is only available to signed in users")
            else:
                is_available = not self.applications.filter(
                    voucher=self, user=user
                ).exists()
                if not is_available:
                    message = _(
                        "You have already used this voucher in a previous order"
                    )
        return is_available, message

    def is_available_for_basket(self, basket):
        """
        Tests whether this voucher is available to the passed basket.

        Returns a tuple of a boolean for whether it is successful, and a
        availability message.
        """
        is_available, message = self.is_available_to_user(user=basket.owner)
        if not is_available:
            return False, message

        is_available, message = False, _(
            "This voucher is not available for this basket"
        )
        for offer in self.offers.all():
            if offer.is_condition_satisfied(basket=basket):
                is_available = True
                message = ""
                break
        return is_available, message

    def record_usage(self, order, user):
        """
        Records a usage of this voucher in an order.
        """
        if user.is_authenticated:
            self.applications.create(voucher=self, order=order, user=user)
        else:
            self.applications.create(voucher=self, order=order)
        self.num_orders += 1
        self.save()

    record_usage.alters_data = True

    def record_discount(self, discount):
        """
        Record a discount that this offer has given
        """
        self.total_discount += discount["discount"]
        self.save()

    record_discount.alters_data = True

    @property
    @deprecated
    def benefit(self):
        """
        Returns the first offer's benefit instance.

        A voucher is commonly only linked to one offer. In that case,
        this helper can be used for convenience.
        """
        return self.offers.first().benefit


class AbstractVoucherApplication(models.Model):
    """
    For tracking how often a voucher has been used in an order.

    This is used to enforce the voucher usage mode in
    Voucher.is_available_to_user, and created in Voucher.record_usage.
    """

    voucher = models.ForeignKey(
        "voucher.Voucher",
        on_delete=models.CASCADE,
        related_name="applications",
        verbose_name=_("Voucher"),
    )

    # It is possible for an anonymous user to apply a voucher so we need to
    # allow the user to be nullable
    user = models.ForeignKey(
        AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        verbose_name=_("User"),
    )
    order = models.ForeignKey(
        "order.Order", on_delete=models.CASCADE, verbose_name=_("Order")
    )
    date_created = models.DateTimeField(auto_now_add=True, db_index=True)

    class Meta:
        abstract = True
        app_label = "voucher"
        ordering = ["-date_created"]
        verbose_name = _("Voucher Application")
        verbose_name_plural = _("Voucher Applications")

    def __str__(self):
        return _("'%(voucher)s' used by '%(user)s'") % {
            "voucher": self.voucher,
            "user": self.user,
        }



from django.contrib import messages
from django.contrib.sites.shortcuts import get_current_site
from django.template import TemplateSyntaxError
from django.utils.translation import gettext_lazy as _
from django.views import generic

from oscar.core.loading import get_class, get_model

CommunicationEventType = get_model("communication", "CommunicationEventType")
CommunicationEventTypeForm = get_class(
    "dashboard.communications.forms", "CommunicationEventTypeForm"
)
Dispatcher = get_class("communication.utils", "Dispatcher")


class ListView(generic.ListView):
    model = CommunicationEventType
    template_name = "oscar/dashboard/comms/list.html"
    context_object_name = "commtypes"


class UpdateView(generic.UpdateView):
    model = CommunicationEventType
    form_class = CommunicationEventTypeForm
    template_name = "oscar/dashboard/comms/detail.html"
    context_object_name = "commtype"
    success_url = "."
    slug_field = "code"

    def form_invalid(self, form):
        messages.error(
            self.request,
            _(
                "The submitted form was not valid, please correct "
                "the errors and resubmit"
            ),
        )
        return super().form_invalid(form)

    def form_valid(self, form):
        if "send_preview" in self.request.POST:
            return self.send_preview(form)
        if "show_preview" in self.request.POST:
            return self.show_preview(form)
        messages.success(self.request, _("Email saved"))
        return super().form_valid(form)

    def get_messages_context(self, form):
        ctx = {"user": self.request.user, "site": get_current_site(self.request)}
        ctx.update(form.get_preview_context())
        return ctx

    def show_preview(self, form):
        ctx = super().get_context_data()
        ctx["form"] = form

        commtype = form.save(commit=False)
        commtype_ctx = self.get_messages_context(form)
        try:
            msgs = commtype.get_messages(commtype_ctx)
        except TemplateSyntaxError as e:
            form.errors["__all__"] = form.error_class([str(e)])
            return self.render_to_response(ctx)

        ctx["show_preview"] = True
        ctx["preview"] = msgs
        return self.render_to_response(ctx)

    def send_preview(self, form):
        ctx = super().get_context_data()
        ctx["form"] = form

        commtype = form.save(commit=False)
        commtype_ctx = self.get_messages_context(form)
        try:
            msgs = commtype.get_messages(commtype_ctx)
        except TemplateSyntaxError as e:
            form.errors["__all__"] = form.error_class([str(e)])
            return self.render_to_response(ctx)

        email = form.cleaned_data["preview_email"]
        dispatch = Dispatcher()
        dispatch.send_email_messages(email, msgs)
        messages.success(self.request, _("A preview email has been sent to %s") % email)

        return self.render_to_response(ctx)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-21 14:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0012_auto_20170609_1902'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productattribute',
            name='option_group',
            field=models.ForeignKey(blank=True, help_text='Select an option group if using type "Option" or "Multi Option"', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='product_attributes', to='catalogue.AttributeOptionGroup', verbose_name='Option Group'),
        ),
    ]



from django.db import models

from oscar.checks import use_productcategory_materialised_view
from oscar.core.loading import get_class, get_model

ExpandUpwardsCategoryQueryset = get_class(
    "catalogue.expressions", "ExpandUpwardsCategoryQueryset"
)
ProductCategoryHierarchy = get_model("catalogue", "ProductCategoryHierarchy")


class RangeQuerySet(models.query.QuerySet):
    """
    This queryset add ``contains_product`` which allows selecting the
    ranges that contain the product in question.
    """

    def _excluded_products_clause(self, product):
        if product.structure == product.CHILD:
            # child products are excluded from a range if either they are
            # excluded, or their parent.
            return ~(
                models.Q(excluded_products=product)
                | models.Q(excluded_products__id=product.parent_id)
            )
        return ~models.Q(excluded_products=product)

    def _excluded_categories_clause(self, product):
        if product.structure == product.CHILD:
            # child products are excluded from a range if their parent contains
            # category that is excluded
            return ~(
                models.Q(
                    excluded_categories__id__in=product.parent.categories.values("id")
                )
            )
        return ~models.Q(excluded_categories__id__in=product.categories.values("id"))

    def _included_products_clause(self, product):
        if product.structure == product.CHILD:
            # child products are included in a range if either they are
            # included, or their parent is included
            return models.Q(included_products=product) | models.Q(
                included_products__id=product.parent_id
            )
        else:
            return models.Q(included_products=product)

    def _productclasses_clause(self, product):
        if product.structure == product.CHILD:
            # child products are included in a range if their parent is
            # included in the range by means of their productclass.
            return models.Q(classes__products__parent_id=product.parent_id)
        return models.Q(classes__id=product.product_class_id)

    def _get_category_ids(self, product):
        if product.structure == product.CHILD:
            # Since a child can not be in a category, it must be determined
            # which category the parent is in
            ProductCategory = product.productcategory_set.model
            return ProductCategory.objects.filter(product_id=product.parent_id).values(
                "category_id"
            )

        return product.categories.values("id")

    def get_category_query(self, product):
        if use_productcategory_materialised_view():
            if product.structure == product.CHILD:
                return ProductCategoryHierarchy.objects.filter(
                    product_id=product.parent_id
                ).values_list("category_id")

            return ProductCategoryHierarchy.objects.filter(
                product_id=product.id
            ).values_list("category_id")

        return ExpandUpwardsCategoryQueryset(self._get_category_ids(product))

    def contains_product(self, product):
        # the wide query is used to determine which ranges have includes_all_products
        # turned on, we only need to look at explicit exclusions, the other
        # mechanism for adding a product to a range don't need to be checked
        wide = self.filter(
            self._excluded_products_clause(product),
            self._excluded_categories_clause(product),
            includes_all_products=True,
        )
        narrow = self.filter(
            self._excluded_products_clause(product),
            self._excluded_categories_clause(product),
            self._included_products_clause(product)
            | models.Q(included_categories__in=self.get_category_query(product))
            | self._productclasses_clause(product),
            includes_all_products=False,
        )
        return wide | narrow



# -*- coding: utf-8 -*-
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import (
    CreateView,
    DeleteView,
    FormView,
    ListView,
    UpdateView,
    View,
)

from oscar.core.loading import get_class, get_classes, get_model
from oscar.core.utils import redirect_to_referrer, safe_referrer

WishList = get_model("wishlists", "WishList")
Line = get_model("wishlists", "Line")
Product = get_model("catalogue", "Product")
WishListForm = get_class("wishlists.forms", "WishListForm")
LineFormset, WishListSharedEmailFormset = get_classes(
    "wishlists.formsets", ("LineFormset", "WishListSharedEmailFormset")
)
PageTitleMixin = get_class("customer.mixins", "PageTitleMixin")


class WishListListView(PageTitleMixin, ListView):
    context_object_name = active_tab = "wishlists"
    template_name = "oscar/customer/wishlists/wishlists_list.html"
    page_title = _("Wish Lists")

    def get_queryset(self):
        """
        Return a list of all the wishlists for the currently
        authenticated user.
        """
        return self.request.user.wishlists.all()


class WishListDetailView(PageTitleMixin, FormView):
    """
    This view acts as a DetailView for a wish list and allows updating the
    quantities of products.

    It is implemented as FormView because it's easier to adapt a FormView to
    display a product then adapt a DetailView to handle form validation.
    """

    template_name = "oscar/customer/wishlists/wishlists_detail.html"
    active_tab = "wishlists"
    form_class = LineFormset

    def dispatch(self, request, *args, **kwargs):
        # pylint: disable=attribute-defined-outside-init
        self.object = self.get_wishlist_or_404(kwargs["key"], request.user)
        return super().dispatch(request, *args, **kwargs)

    def get_wishlist_or_404(self, key, user):
        wishlist = get_object_or_404(WishList, key=key)
        if wishlist.is_allowed_to_edit(user):
            return wishlist
        else:
            raise Http404

    def get_page_title(self):
        return self.object.name

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["instance"] = self.object
        return kwargs

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["wishlist"] = self.object
        other_wishlists = self.request.user.wishlists.exclude(pk=self.object.pk)
        ctx["other_wishlists"] = other_wishlists
        return ctx

    def form_valid(self, form):
        for subform in form:
            if subform.cleaned_data["quantity"] <= 0:
                subform.instance.delete()
            else:
                subform.save()
        messages.success(self.request, _("Quantities updated."))
        return redirect("customer:wishlists-detail", key=self.object.key)


class WishListCreateUpdateViewMixin(PageTitleMixin):
    """
    The wishlist create and update view have the same approach on saving the wislist and shared email
    forms. This mixin handles that in the post view, where it will call process_wishlist_forms
    if both forms are valid. If one of the forms is not valid, the user will be redirected to the original
    view with form errors.
    """

    def process_wishlist_forms(self, wishlist_form, shared_emails_formset):
        wishlist = wishlist_form.save()

        for form in shared_emails_formset:
            # Prevents saving empty or unchanged forms in the formset.
            if not form.has_changed():
                continue

            # Don't commit to DB until we saved the wislist instance.
            wishlist_shared_email = form.save(commit=False)
            wishlist_shared_email.wishlist = wishlist
            wishlist_shared_email.save()

        if wishlist.shared_emails.exists() and wishlist.visibility != WishList.SHARED:
            if wishlist.visibility == WishList.PRIVATE:
                msg = _(
                    "The shared accounts won't be able to access your wishlist "
                    "because the visiblity is set to private."
                )
                messages.warning(self.request, msg)
            elif wishlist.visibility == WishList.PUBLIC:
                msg = _(
                    "You have added shared accounts to your wishlist but the visiblity "
                    "is public, this means everyone with a link has access to it."
                )
                messages.warning(self.request, msg)

        return wishlist

    def post(self, request, *args, **kwargs):
        """
        This post method will handle both the create and update view post request.
        """
        try:
            self.object = self.get_object()
        except AttributeError:
            self.object = None

        form = self.get_form()
        shared_emails_formset = WishListSharedEmailFormset(
            request.POST, instance=self.object
        )

        if form.is_valid() and shared_emails_formset.is_valid():
            wishlist = self.process_wishlist_forms(form, shared_emails_formset)
            return self.form_valid(wishlist)

        context = self.get_context_data(
            form=form, shared_emails_formset=shared_emails_formset
        )
        return self.render_to_response(context)


class WishListCreateView(WishListCreateUpdateViewMixin, CreateView):
    """
    Create a new wishlist

    If a product ID is passed as a kwargs, then this product will be added to
    the wishlist.
    """

    model = WishList
    template_name = "oscar/customer/wishlists/wishlists_form.html"
    active_tab = "wishlists"
    page_title = _("Create a new wish list")
    form_class = WishListForm
    product = None

    def dispatch(self, request, *args, **kwargs):
        if "product_pk" in kwargs:
            try:
                self.product = Product.objects.get(pk=kwargs["product_pk"])
            except ObjectDoesNotExist:
                messages.error(request, _("The requested product no longer exists"))
                return redirect("wishlists-create")
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["product"] = self.product

        # Invalid post response passes this to the context.
        if "shared_emails_formset" not in kwargs:
            ctx["shared_emails_formset"] = WishListSharedEmailFormset()

        return ctx

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["user"] = self.request.user
        return kwargs

    def form_valid(self, form):
        """
        The form argument is actually the wishlist instance because we already saved this in
        the post method below. This is also why we do not call form.save() here.
        """
        wishlist = form
        if self.product:
            wishlist.add(self.product)
            msg = _("Your wishlist has been created and '%(name)s has been added") % {
                "name": self.product.get_title()
            }
        else:
            msg = _("Your wishlist has been created")
        messages.success(self.request, msg)

        return redirect(form.get_absolute_url())


class WishListCreateWithProductView(View):
    """
    Create a wish list and immediately add a product to it
    """

    def post(self, request, *args, **kwargs):
        product = get_object_or_404(Product, pk=kwargs["product_pk"])
        wishlists = request.user.wishlists.all()
        if len(wishlists) == 0:
            wishlist = request.user.wishlists.create()
        else:
            # This shouldn't really happen but we default to using the first
            # wishlist for a user if one already exists when they make this
            # request.
            wishlist = wishlists[0]
        wishlist.add(product)
        messages.success(
            request,
            _("%(title)s has been added to your wishlist")
            % {"title": product.get_title()},
        )
        return redirect_to_referrer(request, wishlist.get_absolute_url())


class WishListUpdateView(WishListCreateUpdateViewMixin, UpdateView):
    model = WishList
    template_name = "oscar/customer/wishlists/wishlists_form.html"
    active_tab = "wishlists"
    form_class = WishListForm
    context_object_name = "wishlist"

    def get_page_title(self):
        return self.object.name

    def get_object(self, queryset=None):
        return get_object_or_404(
            WishList, owner=self.request.user, key=self.kwargs["key"]
        )

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["user"] = self.request.user
        return kwargs

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)

        # Invalid post response passes this to the context.
        if "shared_emails_formset" not in kwargs:
            ctx["shared_emails_formset"] = WishListSharedEmailFormset(
                instance=self.object
            )

        return ctx

    def get_success_url(self):
        messages.success(
            self.request, _("Your '%s' wishlist has been updated") % self.object.name
        )
        return reverse("customer:wishlists-list")

    def form_valid(self, form):
        return redirect(self.get_success_url())


class WishListDeleteView(PageTitleMixin, DeleteView):
    model = WishList
    template_name = "oscar/customer/wishlists/wishlists_delete.html"
    active_tab = "wishlists"

    def get_page_title(self):
        return _("Delete %s") % self.object.name

    def get_object(self, queryset=None):
        return get_object_or_404(
            WishList, owner=self.request.user, key=self.kwargs["key"]
        )

    def get_success_url(self):
        messages.success(
            self.request, _("Your '%s' wish list has been deleted") % self.object.name
        )
        return reverse("customer:wishlists-list")


class WishListAddProduct(View):
    """
    Adds a product to a wish list.

    - If the user doesn't already have a wishlist then it will be created for
      them.
    - If the product is already in the wish list, its quantity is increased.
    """

    # pylint: disable=attribute-defined-outside-init
    def dispatch(self, request, *args, **kwargs):
        self.product = get_object_or_404(Product, pk=kwargs["product_pk"])
        self.wishlist = self.get_or_create_wishlist(request, *args, **kwargs)
        return super().dispatch(request)

    def get_or_create_wishlist(self, request, *args, **kwargs):
        if "key" in kwargs:
            wishlist = get_object_or_404(
                WishList, key=kwargs["key"], owner=request.user
            )
        else:
            wishlists = request.user.wishlists.all()[:1]
            if not wishlists:
                return request.user.wishlists.create()
            wishlist = wishlists[0]

        if not wishlist.is_allowed_to_edit(request.user):
            raise PermissionDenied
        return wishlist

    def get(self, request, *args, **kwargs):
        # This is nasty as we shouldn't be performing write operations on a GET
        # request.  It's only included as the UI of the product detail page
        # allows a wishlist to be selected from a dropdown.
        return self.add_product()

    def post(self, request, *args, **kwargs):
        return self.add_product()

    def add_product(self):
        self.wishlist.add(self.product)
        msg = _("'%s' was added to your wish list.") % self.product.get_title()
        messages.success(self.request, msg)
        return redirect_to_referrer(self.request, self.product.get_absolute_url())


class LineMixin(object):
    """
    Handles fetching both a wish list and a product
    Views using this mixin must be passed two keyword arguments:

    * key: The key of a wish list
    * line_pk: The primary key of the wish list line

    or

    * product_pk: The primary key of the product
    """

    def fetch_line(self, user, wishlist_key, line_pk=None, product_pk=None):
        if line_pk is not None:
            self.line = get_object_or_404(
                Line,
                pk=line_pk,
                wishlist__owner=user,
                wishlist__key=wishlist_key,
            )
        else:
            try:
                self.line = get_object_or_404(
                    Line,
                    product_id=product_pk,
                    wishlist__owner=user,
                    wishlist__key=wishlist_key,
                )
            except Line.MultipleObjectsReturned:
                raise Http404
        self.wishlist = self.line.wishlist
        self.product = self.line.product


class WishListRemoveProduct(LineMixin, PageTitleMixin, DeleteView):
    template_name = "oscar/customer/wishlists/wishlists_delete_product.html"
    active_tab = "wishlists"

    def get_page_title(self):
        return _("Remove %s") % self.object.get_title()

    def get_object(self, queryset=None):
        self.fetch_line(
            self.request.user,
            self.kwargs["key"],
            self.kwargs.get("line_pk"),
            self.kwargs.get("product_pk"),
        )
        return self.line

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["wishlist"] = self.wishlist
        ctx["product"] = self.product
        return ctx

    def get_success_url(self):
        msg = _("'%(title)s' was removed from your '%(name)s' wish list") % {
            "title": self.line.get_title(),
            "name": self.wishlist.name,
        }
        messages.success(self.request, msg)

        # We post directly to this view on product pages; and should send the
        # user back there if that was the case
        referrer = safe_referrer(self.request, "")
        if referrer and self.product and self.product.get_absolute_url() in referrer:
            return referrer
        else:
            return reverse(
                "customer:wishlists-detail", kwargs={"key": self.wishlist.key}
            )


class WishListMoveProductToAnotherWishList(LineMixin, View):
    def dispatch(self, request, *args, **kwargs):
        self.fetch_line(request.user, kwargs["key"], line_pk=kwargs["line_pk"])
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        to_wishlist = get_object_or_404(
            WishList, owner=request.user, key=kwargs["to_key"]
        )

        if to_wishlist.lines.filter(product=self.line.product).count() > 0:
            msg = _("Wish list '%(name)s' already containing '%(title)s'") % {
                "title": self.product.get_title(),
                "name": to_wishlist.name,
            }
            messages.error(self.request, msg)
        else:
            self.line.wishlist = to_wishlist
            self.line.save()

            msg = _("'%(title)s' moved to '%(name)s' wishlist") % {
                "title": self.product.get_title(),
                "name": to_wishlist.name,
            }
            messages.success(self.request, msg)

        default_url = reverse(
            "customer:wishlists-detail", kwargs={"key": self.wishlist.key}
        )
        return redirect_to_referrer(self.request, default_url)



from django.contrib import admin

from oscar.core.loading import get_model

Email = get_model("communication", "Email")
Notification = get_model("communication", "Notification")
CommunicationEventType = get_model("communication", "CommunicationEventType")

admin.site.register(Email)
admin.site.register(Notification)
admin.site.register(CommunicationEventType)



# Generated by Django 2.0.7 on 2018-11-15 19:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('voucher', '0006_auto_20180413_0911'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='end_datetime',
            field=models.DateTimeField(db_index=True, verbose_name='End datetime'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='start_datetime',
            field=models.DateTimeField(db_index=True, verbose_name='Start datetime'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import django.core.validators
from django.utils.module_loading import import_string
from django.conf import settings
import oscar.models.fields.autoslugfield
import oscar.models.fields

models_AutoField = import_string(settings.DEFAULT_AUTO_FIELD)


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AttributeOption',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('option', models.CharField(max_length=255, verbose_name='Option')),
            ],
            options={
                'verbose_name_plural': 'Attribute options',
                'verbose_name': 'Attribute option',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AttributeOptionGroup',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
            ],
            options={
                'verbose_name_plural': 'Attribute option groups',
                'verbose_name': 'Attribute option group',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(unique=True, max_length=255)),
                ('depth', models.PositiveIntegerField()),
                ('numchild', models.PositiveIntegerField(default=0)),
                ('name', models.CharField(max_length=255, db_index=True, verbose_name='Name')),
                ('description', models.TextField(verbose_name='Description', blank=True)),
                ('image', models.ImageField(upload_to='categories', verbose_name='Image', max_length=255, blank=True, null=True)),
                ('slug', models.SlugField(max_length=255, editable=False, verbose_name='Slug')),
                ('full_name', models.CharField(max_length=255, editable=False, db_index=True, verbose_name='Full Name')),
            ],
            options={
                'ordering': ['full_name'],
                'verbose_name_plural': 'Categories',
                'verbose_name': 'Category',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(populate_from='name', unique=True, verbose_name='Code', max_length=128, editable=False, blank=True)),
                ('type', models.CharField(default='Required', max_length=128, verbose_name='Status', choices=[('Required', 'Required - a value for this option must be specified'), ('Optional', 'Optional - a value for this option can be omitted')])),
            ],
            options={
                'verbose_name_plural': 'Options',
                'verbose_name': 'Option',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('structure', models.CharField(default='standalone', max_length=10, verbose_name='Product structure', choices=[('standalone', 'Stand-alone product'), ('parent', 'Parent product'), ('child', 'Child product')])),
                ('upc', oscar.models.fields.NullCharField(unique=True, verbose_name='UPC', max_length=64, help_text='Universal Product Code (UPC) is an identifier for a product which is not specific to a particular  supplier. Eg an ISBN for a book.')),
                ('title', models.CharField(max_length=255, verbose_name='Title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('description', models.TextField(verbose_name='Description', blank=True)),
                ('rating', models.FloatField(editable=False, verbose_name='Rating', null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('date_updated', models.DateTimeField(auto_now=True, db_index=True, verbose_name='Date updated')),
                ('is_discountable', models.BooleanField(default=True, verbose_name='Is discountable?', help_text='This flag indicates if this product can be used in an offer or not')),
            ],
            options={
                'ordering': ['-date_created'],
                'verbose_name_plural': 'Products',
                'verbose_name': 'Product',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductAttribute',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('code', models.SlugField(max_length=128, verbose_name='Code', validators=[django.core.validators.RegexValidator(regex='^[a-zA-Z\\-_][0-9a-zA-Z\\-_]*$', message="Code can only contain the letters a-z, A-Z, digits, minus and underscores, and can't start with a digit")])),
                ('type', models.CharField(default='text', max_length=20, verbose_name='Type', choices=[('text', 'Text'), ('integer', 'Integer'), ('boolean', 'True / False'), ('float', 'Float'), ('richtext', 'Rich Text'), ('date', 'Date'), ('option', 'Option'), ('entity', 'Entity'), ('file', 'File'), ('image', 'Image')])),
                ('required', models.BooleanField(default=False, verbose_name='Required')),
                ('option_group', models.ForeignKey(null=True, verbose_name='Option Group', help_text='Select an option group if using type "Option"', to='catalogue.AttributeOptionGroup', blank=True, on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ['code'],
                'verbose_name_plural': 'Product attributes',
                'verbose_name': 'Product attribute',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductAttributeValue',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value_text', models.TextField(blank=True, verbose_name='Text', null=True)),
                ('value_integer', models.IntegerField(blank=True, verbose_name='Integer', null=True)),
                ('value_boolean', models.NullBooleanField(verbose_name='Boolean')),
                ('value_float', models.FloatField(blank=True, verbose_name='Float', null=True)),
                ('value_richtext', models.TextField(blank=True, verbose_name='Richtext', null=True)),
                ('value_date', models.DateField(blank=True, verbose_name='Date', null=True)),
                ('value_file', models.FileField(upload_to='images/products/%Y/%m/', max_length=255, blank=True, null=True)),
                ('value_image', models.ImageField(upload_to='images/products/%Y/%m/', max_length=255, blank=True, null=True)),
                ('entity_object_id', models.PositiveIntegerField(blank=True, editable=False, null=True)),
                ('attribute', models.ForeignKey(verbose_name='Attribute', to='catalogue.ProductAttribute', on_delete=models.CASCADE)),
                ('entity_content_type', models.ForeignKey(null=True, editable=False, to='contenttypes.ContentType', blank=True, on_delete=models.CASCADE)),
                ('product', models.ForeignKey(verbose_name='Product', related_name='attribute_values', to='catalogue.Product', on_delete=models.CASCADE)),
                ('value_option', models.ForeignKey(null=True, verbose_name='Value option', to='catalogue.AttributeOption', blank=True, on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name_plural': 'Product attribute values',
                'verbose_name': 'Product attribute value',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductCategory',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.ForeignKey(verbose_name='Category', to='catalogue.Category', on_delete=models.CASCADE)),
                ('product', models.ForeignKey(verbose_name='Product', to='catalogue.Product', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ['product', 'category'],
                'verbose_name_plural': 'Product categories',
                'verbose_name': 'Product category',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductClass',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', oscar.models.fields.autoslugfield.AutoSlugField(populate_from='name', unique=True, verbose_name='Slug', max_length=128, editable=False, blank=True)),
                ('requires_shipping', models.BooleanField(default=True, verbose_name='Requires shipping?')),
                ('track_stock', models.BooleanField(default=True, verbose_name='Track stock levels?')),
                ('options', models.ManyToManyField(verbose_name='Options', to='catalogue.Option', blank=True)),
            ],
            options={
                'ordering': ['name'],
                'verbose_name_plural': 'Product classes',
                'verbose_name': 'Product class',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductImage',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('original', models.ImageField(upload_to='images/products/%Y/%m/', max_length=255, verbose_name='Original')),
                ('caption', models.CharField(max_length=200, verbose_name='Caption', blank=True)),
                ('display_order', models.PositiveIntegerField(default=0, verbose_name='Display order', help_text='An image with a display order of zero will be the primary image for a product')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('product', models.ForeignKey(verbose_name='Product', related_name='images', to='catalogue.Product', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ['display_order'],
                'verbose_name_plural': 'Product images',
                'verbose_name': 'Product image',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProductRecommendation',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ranking', models.PositiveSmallIntegerField(default=0, verbose_name='Ranking', help_text='Determines order of the products. A product with a higher value will appear before one with a lower ranking.')),
                ('primary', models.ForeignKey(verbose_name='Primary product', related_name='primary_recommendations', to='catalogue.Product', on_delete=models.CASCADE)),
                ('recommendation', models.ForeignKey(verbose_name='Recommended product', to='catalogue.Product', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ['primary', '-ranking'],
                'verbose_name_plural': 'Product recomendations',
                'verbose_name': 'Product recommendation',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='productrecommendation',
            unique_together=set([('primary', 'recommendation')]),
        ),
        migrations.AlterUniqueTogether(
            name='productimage',
            unique_together=set([('product', 'display_order')]),
        ),
        migrations.AlterUniqueTogether(
            name='productcategory',
            unique_together=set([('product', 'category')]),
        ),
        migrations.AlterUniqueTogether(
            name='productattributevalue',
            unique_together=set([('attribute', 'product')]),
        ),
        migrations.AddField(
            model_name='productattribute',
            name='product_class',
            field=models.ForeignKey(null=True, verbose_name='Product type', related_name='attributes', to='catalogue.ProductClass', blank=True, on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='attributes',
            field=models.ManyToManyField(verbose_name='Attributes', help_text='A product attribute is something that this product may have, such as a size, as specified by its class', to='catalogue.ProductAttribute', through='catalogue.ProductAttributeValue'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='categories',
            field=models.ManyToManyField(through='catalogue.ProductCategory', verbose_name='Categories', to='catalogue.Category'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='parent',
            field=models.ForeignKey(null=True, verbose_name='Parent product', related_name='children', help_text="Only choose a parent product if you're creating a child product.  For example if this is a size 4 of a particular t-shirt.  Leave blank if this is a stand-alone product (i.e. there is only one version of this product).", to='catalogue.Product', blank=True, on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='product_class',
            field=models.ForeignKey(verbose_name='Product type', on_delete=django.db.models.deletion.PROTECT, related_name='products', help_text='Choose what type of product this is', to='catalogue.ProductClass', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='product_options',
            field=models.ManyToManyField(verbose_name='Product options', help_text="Options are values that can be associated with a item when it is added to a customer's basket.  This could be something like a personalised message to be printed on a T-shirt.", to='catalogue.Option', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='product',
            name='recommended_products',
            field=models.ManyToManyField(verbose_name='Recommended products', help_text='These are products that are recommended to accompany the main product.', to='catalogue.Product', through='catalogue.ProductRecommendation', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='attributeoption',
            name='group',
            field=models.ForeignKey(verbose_name='Group', related_name='options', to='catalogue.AttributeOptionGroup', on_delete=models.CASCADE),
            preserve_default=True,
        ),
    ]



from itertools import zip_longest

from django.utils.crypto import get_random_string

from oscar.core.loading import get_model


def generate_code(
    length, chars="ABCDEFGHJKLMNPQRSTUVWXYZ23456789", group_length=4, separator="-"
):
    """Create a string of 16 chars grouped by 4 chars."""
    random_string = (i for i in get_random_string(length=length, allowed_chars=chars))
    return separator.join(
        "".join(filter(None, a)) for a in zip_longest(*[random_string] * group_length)
    )


def get_unused_code(length=12, group_length=4, separator="-"):
    """Generate a code, check in the db if it already exists and return it.

    i.e. ASDA-QWEE-DFDF-KFGG

    :param int length: the number of characters in the code
    :param int group_length: length of character groups separated by separator kwarg ('-' by default)
    :param str separator: separator string for voucher code
    :return: voucher code
    :rtype: str

    """
    Voucher = get_model("voucher", "Voucher")
    while True:
        code = generate_code(length, group_length=group_length, separator=separator)
        if not Voucher.objects.filter(code=code).exists():
            return code



from django import forms
from django.conf import settings

from oscar.core.loading import get_model
from oscar.forms.mixins import PhoneNumberMixin

UserAddress = get_model("address", "useraddress")


class AbstractAddressForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        """
        Set fields in OSCAR_REQUIRED_ADDRESS_FIELDS as required.
        """
        super().__init__(*args, **kwargs)
        field_names = set(self.fields) & set(settings.OSCAR_REQUIRED_ADDRESS_FIELDS)
        for field_name in field_names:
            self.fields[field_name].required = True


class UserAddressForm(PhoneNumberMixin, AbstractAddressForm):
    class Meta:
        model = UserAddress
        fields = [
            "first_name",
            "last_name",
            "line1",
            "line2",
            "line3",
            "line4",
            "state",
            "postcode",
            "country",
            "phone_number",
            "notes",
        ]

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instance.user = user



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-04-15 14:18
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0003_auto_20161120_1707'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='conditionaloffer',
            options={'ordering': ['-priority', 'pk'], 'verbose_name': 'Conditional offer', 'verbose_name_plural': 'Conditional offers'},
        ),
    ]



from django.contrib.auth.decorators import login_required
from django.urls import path
from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarConfig
from oscar.core.loading import get_class


class CatalogueReviewsConfig(OscarConfig):
    label = "reviews"
    name = "oscar.apps.catalogue.reviews"
    verbose_name = _("Catalogue reviews")

    hidable_feature_name = "reviews"

    # pylint: disable=attribute-defined-outside-init
    def ready(self):
        self.detail_view = get_class("catalogue.reviews.views", "ProductReviewDetail")
        self.create_view = get_class("catalogue.reviews.views", "CreateProductReview")
        self.vote_view = get_class("catalogue.reviews.views", "AddVoteView")
        self.list_view = get_class("catalogue.reviews.views", "ProductReviewList")

    def get_urls(self):
        urls = [
            path("<int:pk>/", self.detail_view.as_view(), name="reviews-detail"),
            path("add/", self.create_view.as_view(), name="reviews-add"),
            path(
                "<int:pk>/vote/",
                login_required(self.vote_view.as_view()),
                name="reviews-vote",
            ),
            path("", self.list_view.as_view(), name="reviews-list"),
        ]
        return self.post_process_urls(urls)



import logging

from django.core.management.base import BaseCommand

from oscar.core.loading import get_class

Calculator = get_class("analytics.scores", "Calculator")

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Calculate product scores based on analytics data"

    def handle(self, *args, **options):
        Calculator(logger).run()



from django.urls import path
from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarConfig
from oscar.core.loading import get_class


class SearchConfig(OscarConfig):
    label = "search"
    name = "oscar.apps.search"
    verbose_name = _("Search")

    namespace = "search"

    # pylint: disable=attribute-defined-outside-init
    def ready(self):
        self.search_view = get_class("search.views", "FacetedSearchView")

    def get_urls(self):
        urlpatterns = [path("", self.search_view.as_view(), name="search")]

        return self.post_process_urls(urlpatterns)



from django.urls import path
from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarDashboardConfig
from oscar.core.loading import get_class


class OrdersDashboardConfig(OscarDashboardConfig):
    label = "orders_dashboard"
    name = "oscar.apps.dashboard.orders"
    verbose_name = _("Orders dashboard")

    default_permissions = [
        "is_staff",
    ]

    def configure_permissions(self):
        DashboardPermission = get_class("dashboard.permissions", "DashboardPermission")

        self.permissions_map = {
            "order-list": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
            "order-stats": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
            "order-detail": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
            "order-detail-note": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
            "order-line-detail": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
            "order-shipping-address": (
                DashboardPermission.get("order"),
                DashboardPermission.partner_dashboard_access,
            ),
        }

    # pylint: disable=attribute-defined-outside-init
    def ready(self):
        self.order_list_view = get_class("dashboard.orders.views", "OrderListView")
        self.order_detail_view = get_class("dashboard.orders.views", "OrderDetailView")
        self.shipping_address_view = get_class(
            "dashboard.orders.views", "ShippingAddressUpdateView"
        )
        self.line_detail_view = get_class("dashboard.orders.views", "LineDetailView")
        self.order_stats_view = get_class("dashboard.orders.views", "OrderStatsView")
        self.configure_permissions()

    def get_urls(self):
        urls = [
            path("", self.order_list_view.as_view(), name="order-list"),
            path("statistics/", self.order_stats_view.as_view(), name="order-stats"),
            path(
                "<str:number>/", self.order_detail_view.as_view(), name="order-detail"
            ),
            path(
                "<str:number>/notes/<int:note_id>/",
                self.order_detail_view.as_view(),
                name="order-detail-note",
            ),
            path(
                "<str:number>/lines/<int:line_id>/",
                self.line_detail_view.as_view(),
                name="order-line-detail",
            ),
            path(
                "<str:number>/shipping-address/",
                self.shipping_address_view.as_view(),
                name="order-shipping-address",
            ),
        ]
        return self.post_process_urls(urls)



from decimal import Decimal as D

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _

from oscar.core.loading import get_classes

(
    Free,
    NoShippingRequired,
    TaxExclusiveOfferDiscount,
    TaxInclusiveOfferDiscount,
) = get_classes(
    "shipping.methods",
    [
        "Free",
        "NoShippingRequired",
        "TaxExclusiveOfferDiscount",
        "TaxInclusiveOfferDiscount",
    ],
)


class Repository(object):
    """
    Repository class responsible for returning ShippingMethod
    objects for a given user, basket etc
    """

    # We default to just free shipping. Customise this class and override this
    # property to add your own shipping methods. This should be a list of
    # instantiated shipping methods.
    methods = (Free(),)

    # API

    def get_shipping_methods(self, basket, shipping_addr=None, **kwargs):
        """
        Return a list of all applicable shipping method instances for a given
        basket, address etc.
        """
        if not basket.is_shipping_required():
            # Special case! Baskets that don't require shipping get a special
            # shipping method.
            return [NoShippingRequired()]

        methods = self.get_available_shipping_methods(
            basket=basket, shipping_addr=shipping_addr, **kwargs
        )
        if basket.has_shipping_discounts:
            methods = self.apply_shipping_offers(basket, methods)
        return methods

    def get_default_shipping_method(self, basket, shipping_addr=None, **kwargs):
        """
        Return a 'default' shipping method to show on the basket page to give
        the customer an indication of what their order will cost.
        """
        shipping_methods = self.get_shipping_methods(
            basket, shipping_addr=shipping_addr, **kwargs
        )
        if len(shipping_methods) == 0:
            raise ImproperlyConfigured(_("You need to define some shipping methods"))

        # Assume first returned method is default
        return shipping_methods[0]

    # Helpers

    # pylint: disable=unused-argument
    def get_available_shipping_methods(self, basket, shipping_addr=None, **kwargs):
        """
        Return a list of all applicable shipping method instances for a given
        basket, address etc. This method is intended to be overridden.
        """
        return self.methods

    def apply_shipping_offers(self, basket, methods):
        """
        Apply shipping offers to the passed set of methods
        """
        # We default to only applying the first shipping discount.
        offer = basket.shipping_discounts[0]["offer"]
        return [self.apply_shipping_offer(basket, method, offer) for method in methods]

    def apply_shipping_offer(self, basket, method, offer):
        """
        Wrap a shipping method with an offer discount wrapper (as long as the
        shipping charge is non-zero).
        """
        # If the basket has qualified for shipping discount, wrap the shipping
        # method with a decorating class that applies the offer discount to the
        # shipping charge.
        charge = method.calculate(basket)
        if charge.excl_tax == D("0.00"):
            # No need to wrap zero shipping charges
            return method

        if charge.is_tax_known:
            return TaxInclusiveOfferDiscount(method, offer)
        else:
            # When returning a tax exclusive discount, it is assumed
            # that this will be used to calculate taxes which will then
            # be assigned directly to the method instance.
            return TaxExclusiveOfferDiscount(method, offer)



import datetime
import posixpath

from django.conf import settings


# pylint: disable=unused-argument
def get_image_upload_path(instance, filename):
    return posixpath.join(
        datetime.datetime.now().strftime(settings.OSCAR_IMAGE_FOLDER), filename
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-18 10:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import oscar.models.fields.autoslugfield
from django.utils.module_loading import import_string
from django.conf import settings

models_AutoField = import_string(settings.DEFAULT_AUTO_FIELD)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('catalogue', '0013_auto_20170821_1548'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('customer', '0005_auto_20181115_1953'),
    ]

    state_operations = [
        migrations.CreateModel(
            name='CommunicationEventType',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, help_text='Code used for looking up this event programmatically', max_length=128, populate_from='name', separator='_', unique=True, validators=[django.core.validators.RegexValidator(message="Code can only contain the letters a-z, A-Z, digits, and underscores, and can't start with a digit.", regex='^[a-zA-Z_][0-9a-zA-Z_]*$')], verbose_name='Code')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('category', models.CharField(choices=[('Order related', 'Order related'), ('User related', 'User related')], default='Order related', max_length=255, verbose_name='Category')),
                ('email_subject_template', models.CharField(blank=True, max_length=255, null=True, verbose_name='Email Subject Template')),
                ('email_body_template', models.TextField(blank=True, null=True, verbose_name='Email Body Template')),
                ('email_body_html_template', models.TextField(blank=True, help_text='HTML template', null=True, verbose_name='Email Body HTML Template')),
                ('sms_template', models.CharField(blank=True, help_text='SMS template', max_length=170, null=True, verbose_name='SMS Template')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
            ],
            options={
                'verbose_name': 'Communication event type',
                'verbose_name_plural': 'Communication event types',
                'db_table': 'customer_communicationeventtype',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Email Address')),
                ('subject', models.TextField(max_length=255, verbose_name='Subject')),
                ('body_text', models.TextField(verbose_name='Body Text')),
                ('body_html', models.TextField(blank=True, verbose_name='Body HTML')),
                ('date_sent', models.DateTimeField(auto_now_add=True, verbose_name='Date Sent')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='emails', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Email',
                'verbose_name_plural': 'Emails',
                'db_table': 'customer_email',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=255)),
                ('body', models.TextField()),
                ('category', models.CharField(blank=True, max_length=255)),
                ('location', models.CharField(choices=[('Inbox', 'Inbox'), ('Archive', 'Archive')], default='Inbox', max_length=32)),
                ('date_sent', models.DateTimeField(auto_now_add=True)),
                ('date_read', models.DateTimeField(blank=True, null=True)),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
                ('sender', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
                'db_table': 'customer_notification',
                'ordering': ('-date_sent',),
                'abstract': False,
            },
        ),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=state_operations
        ),
    ]



# Generated by Django 3.2.18 on 2023-03-14 11:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0010_conditionaloffer_combinations'),
    ]

    operations = [
        migrations.AddField(
            model_name='rangeproductfileupload',
            name='upload_type',
            field=models.CharField(choices=[('included', 'Included products upload'), ('excluded', 'Excluded products upload')], default='included', max_length=8),
        ),
    ]



import os
import shutil
import tarfile
import tempfile
import zipfile
import zlib

from django.core.exceptions import FieldError
from django.core.files import File
from django.db.transaction import atomic
from django.utils.translation import gettext_lazy as _
from PIL import Image

from oscar.apps.catalogue.exceptions import (
    IdenticalImageError,
    ImageImportError,
    InvalidImageArchive,
)
from oscar.core.loading import get_model

Product = get_model("catalogue", "product")
ProductImage = get_model("catalogue", "productimage")


# This is an old class only really intended to be used by the internal sandbox
# site. It's not recommended to be used by your project.
class Importer(object):
    allowed_extensions = [".jpeg", ".jpg", ".gif", ".png"]

    def __init__(self, logger, field):
        self.logger = logger
        self._field = field

    @atomic
    def handle(self, dirname):
        stats = {"num_processed": 0, "num_skipped": 0, "num_invalid": 0}
        image_dir, filenames = self._get_image_files(dirname)
        if image_dir:
            for filename in filenames:
                try:
                    lookup_value = self._get_lookup_value_from_filename(filename)
                    self._process_image(image_dir, filename, lookup_value)
                    stats["num_processed"] += 1
                except Product.MultipleObjectsReturned:
                    self.logger.warning(
                        "Multiple products matching %s='%s',"
                        " skipping" % (self._field, lookup_value)
                    )
                    stats["num_skipped"] += 1
                except Product.DoesNotExist:
                    self.logger.warning(
                        "No item matching %s='%s'" % (self._field, lookup_value)
                    )
                    stats["num_skipped"] += 1
                except IdenticalImageError:
                    self.logger.warning(
                        "Identical image already exists for"
                        " %s='%s', skipping" % (self._field, lookup_value)
                    )
                    stats["num_skipped"] += 1
                except IOError as e:
                    stats["num_invalid"] += 1
                    raise ImageImportError(
                        _("%(filename)s is not a valid image (%(error)s)")
                        % {"filename": filename, "error": e}
                    )
                except FieldError as e:
                    raise ImageImportError(e)
            if image_dir != dirname:
                shutil.rmtree(image_dir)
        else:
            raise InvalidImageArchive(_("%s is not a valid image archive") % dirname)
        self.logger.info(
            "Finished image import: %(num_processed)d imported,"
            " %(num_skipped)d skipped" % stats
        )

    def _get_image_files(self, dirname):
        filenames = []
        image_dir = self._extract_images(dirname)
        if image_dir:
            for filename in os.listdir(image_dir):
                ext = os.path.splitext(filename)[1]
                if (
                    os.path.isfile(os.path.join(image_dir, filename))
                    and ext in self.allowed_extensions
                ):
                    filenames.append(filename)
        return image_dir, filenames

    def _extract_images(self, dirname):
        """
        Returns path to directory containing images in dirname if successful.
        Returns empty string if dirname does not exist, or could not be opened.
        Assumes that if dirname is a directory, then it contains images.
        If dirname is an archive (tar/zip file) then the path returned is to a
        temporary directory that should be deleted when no longer required.
        """
        if os.path.isdir(dirname):
            return dirname

        ext = os.path.splitext(dirname)[1]
        if ext in [".gz", ".tar"]:
            image_dir = tempfile.mkdtemp()
            try:
                tar_file = tarfile.open(dirname)
                tar_file.extractall(image_dir)
                tar_file.close()
                return image_dir
            except (tarfile.TarError, zlib.error):
                return ""
        elif ext == ".zip":
            image_dir = tempfile.mkdtemp()
            try:
                zip_file = zipfile.ZipFile(dirname)
                zip_file.extractall(image_dir)
                zip_file.close()
                return image_dir
            except (zlib.error, zipfile.BadZipfile, zipfile.LargeZipFile):
                return ""
        # unknown archive - perhaps this should be treated differently
        return ""

    def _process_image(self, dirname, filename, lookup_value):
        file_path = os.path.join(dirname, filename)
        trial_image = Image.open(file_path)
        trial_image.verify()

        kwargs = {self._field: lookup_value}
        item = Product._default_manager.get(**kwargs)

        new_data = open(file_path, "rb").read()
        next_index = 0
        for existing in item.images.all():
            next_index = existing.display_order + 1
            try:
                if new_data == existing.original.read():
                    raise IdenticalImageError()
            except IOError:
                # File probably doesn't exist
                existing.delete()

        new_file = File(open(file_path, "rb"))
        im = ProductImage(product=item, display_order=next_index)
        im.original.save(filename, new_file, save=False)
        im.save()
        self.logger.debug('Image added to "%s"' % item)

    def _fetch_item(self, filename):
        kwargs = {self._field: self._get_lookup_value_from_filename(filename)}
        return Product._default_manager.get(**kwargs)

    def _get_lookup_value_from_filename(self, filename):
        return os.path.splitext(filename)[0]



from django.db.models.signals import post_save
from django.dispatch import receiver

from oscar.core.loading import get_model

StockAlert = get_model("partner", "StockAlert")
StockRecord = get_model("partner", "StockRecord")


# pylint: disable=unused-argument
@receiver(post_save, sender=StockRecord)
def update_stock_alerts(sender, instance, created, **kwargs):
    """
    Update low-stock alerts
    """
    if created or kwargs.get("raw", False):
        return
    stockrecord = instance
    try:
        alert = StockAlert.objects.get(stockrecord=stockrecord, status=StockAlert.OPEN)
    except StockAlert.DoesNotExist:
        alert = None

    if stockrecord.is_below_threshold and not alert:
        StockAlert.objects.create(
            stockrecord=stockrecord, threshold=stockrecord.low_stock_threshold
        )
    elif not stockrecord.is_below_threshold and alert:
        alert.close()



from django.urls import path
from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarDashboardConfig
from oscar.core.loading import get_class


class ShippingDashboardConfig(OscarDashboardConfig):
    label = "shipping_dashboard"
    name = "oscar.apps.dashboard.shipping"
    verbose_name = _("Shipping dashboard")

    default_permissions = ["is_staff"]

    def configure_permissions(self):
        DashboardPermission = get_class("dashboard.permissions", "DashboardPermission")

        self.permissions_map = {
            "shipping-method-list": DashboardPermission.get("shipping_method"),
            "shipping-method-create": DashboardPermission.get("shipping_method"),
            "shipping-method-detail": DashboardPermission.get("shipping_method"),
            "shipping-method-edit": DashboardPermission.get("shipping_method"),
            "shipping-method-delete": DashboardPermission.get("shipping_method"),
            "shipping-method-band-edit": DashboardPermission.get("shipping_method"),
            "shipping-method-band-delete": DashboardPermission.get("shipping_method"),
        }

    # pylint: disable=attribute-defined-outside-init
    def ready(self):
        self.weight_method_list_view = get_class(
            "dashboard.shipping.views", "WeightBasedListView"
        )
        self.weight_method_create_view = get_class(
            "dashboard.shipping.views", "WeightBasedCreateView"
        )
        self.weight_method_edit_view = get_class(
            "dashboard.shipping.views", "WeightBasedUpdateView"
        )
        self.weight_method_delete_view = get_class(
            "dashboard.shipping.views", "WeightBasedDeleteView"
        )
        # This doubles as the weight_band create view
        self.weight_method_detail_view = get_class(
            "dashboard.shipping.views", "WeightBasedDetailView"
        )
        self.weight_band_edit_view = get_class(
            "dashboard.shipping.views", "WeightBandUpdateView"
        )
        self.weight_band_delete_view = get_class(
            "dashboard.shipping.views", "WeightBandDeleteView"
        )
        self.configure_permissions()

    def get_urls(self):
        urlpatterns = [
            path(
                "weight-based/",
                self.weight_method_list_view.as_view(),
                name="shipping-method-list",
            ),
            path(
                "weight-based/create/",
                self.weight_method_create_view.as_view(),
                name="shipping-method-create",
            ),
            path(
                "weight-based/<int:pk>/",
                self.weight_method_detail_view.as_view(),
                name="shipping-method-detail",
            ),
            path(
                "weight-based/<int:pk>/edit/",
                self.weight_method_edit_view.as_view(),
                name="shipping-method-edit",
            ),
            path(
                "weight-based/<int:pk>/delete/",
                self.weight_method_delete_view.as_view(),
                name="shipping-method-delete",
            ),
            path(
                "weight-based/<int:method_pk>/bands/<int:pk>/",
                self.weight_band_edit_view.as_view(),
                name="shipping-method-band-edit",
            ),
            path(
                "weight-based/<int:method_pk>/bands/<int:pk>/delete/",
                self.weight_band_delete_view.as_view(),
                name="shipping-method-band-delete",
            ),
        ]
        return self.post_process_urls(urlpatterns)



from oscar.core.loading import get_class, get_classes

OrderReportGenerator = get_class("order.reports", "OrderReportGenerator")
ProductReportGenerator, UserReportGenerator = get_classes(
    "analytics.reports", ["ProductReportGenerator", "UserReportGenerator"]
)
OpenBasketReportGenerator, SubmittedBasketReportGenerator = get_classes(
    "basket.reports", ["OpenBasketReportGenerator", "SubmittedBasketReportGenerator"]
)
OfferReportGenerator = get_class("offer.reports", "OfferReportGenerator")
VoucherReportGenerator = get_class("voucher.reports", "VoucherReportGenerator")


class GeneratorRepository(object):
    generators = [
        OrderReportGenerator,
        ProductReportGenerator,
        UserReportGenerator,
        OpenBasketReportGenerator,
        SubmittedBasketReportGenerator,
        VoucherReportGenerator,
        OfferReportGenerator,
    ]

    def get_report_generators(self):
        return self.generators

    def get_generator(self, code):
        for generator in self.generators:
            if generator.code == code:
                return generator
        return None



from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarConfig


class AddressConfig(OscarConfig):
    label = "address"
    name = "oscar.apps.address"
    verbose_name = _("Address")



from django.urls import path
from django.utils.translation import gettext_lazy as _

from oscar.core.application import OscarDashboardConfig
from oscar.core.loading import get_class


class RangesDashboardConfig(OscarDashboardConfig):
    label = "ranges_dashboard"
    name = "oscar.apps.dashboard.ranges"
    verbose_name = _("Ranges dashboard")

    default_permissions = [
        "is_staff",
    ]

    def configure_permissions(self):
        DashboardPermission = get_class("dashboard.permissions", "DashboardPermission")

        self.permissions_map = {
            "range-list": DashboardPermission.get("range"),
            "range-create": DashboardPermission.get("range"),
            "range-update": DashboardPermission.get("range"),
            "range-delete": DashboardPermission.get("range"),
            "range-products": DashboardPermission.get("range"),
            "range-reorder": DashboardPermission.get("range"),
        }

    # pylint: disable=attribute-defined-outside-init
    def ready(self):
        self.list_view = get_class("dashboard.ranges.views", "RangeListView")
        self.create_view = get_class("dashboard.ranges.views", "RangeCreateView")
        self.update_view = get_class("dashboard.ranges.views", "RangeUpdateView")
        self.delete_view = get_class("dashboard.ranges.views", "RangeDeleteView")
        self.products_view = get_class("dashboard.ranges.views", "RangeProductListView")
        self.reorder_view = get_class("dashboard.ranges.views", "RangeReorderView")
        self.configure_permissions()

    def get_urls(self):
        urlpatterns = [
            path("", self.list_view.as_view(), name="range-list"),
            path("create/", self.create_view.as_view(), name="range-create"),
            path("<int:pk>/", self.update_view.as_view(), name="range-update"),
            path("<int:pk>/delete/", self.delete_view.as_view(), name="range-delete"),
            path(
                "<int:pk>/products/",
                self.products_view.as_view(),
                name="range-products",
            ),
            path(
                "<int:pk>/reorder/", self.reorder_view.as_view(), name="range-reorder"
            ),
        ]
        return self.post_process_urls(urlpatterns)



# Generated by Django 4.2.5 on 2024-12-04 07:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("catalogue", "0028_product_priority"),
    ]

    operations = [
        migrations.AddField(
            model_name="product",
            name="code",
            field=models.CharField(
                blank=True,
                max_length=255,
                null=True,
                unique=True,
                verbose_name="Code can be used to store a secondary unique identifier, when upc isn't sufficient",
            ),
        ),
    ]



# Generated by Django 2.1.1 on 2018-09-12 08:31

from django.db import migrations, models
import django.db.models.deletion
from django.utils.module_loading import import_string
from django.conf import settings

models_AutoField = import_string(settings.DEFAULT_AUTO_FIELD)


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0005_update_email_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='OrderStatusChange',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('old_status', models.CharField(blank=True, max_length=100, verbose_name='Old Status')),
                ('new_status', models.CharField(blank=True, max_length=100, verbose_name='New Status')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='status_changes', to='order.Order', verbose_name='Order')),
            ],
            options={
                'verbose_name': 'Order Status Change',
                'verbose_name_plural': 'Order Status Changes',
                'ordering': ['-date_created'],
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('basket', '0005_auto_20150604_1450'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='line',
            options={'ordering': ['date_created', 'pk'], 'verbose_name': 'Basket line', 'verbose_name_plural': 'Basket lines'},
        ),
    ]



class SurchargeList(list):
    @property
    def total(self):
        return sum([surcharge.price for surcharge in self])


class SurchargePrice:
    surcharge = None
    price = None

    def __init__(self, surcharge, price):
        self.surcharge = surcharge
        self.price = price


class SurchargeApplicator:
    def __init__(self, request=None, context=None):
        self.context = context
        self.request = request

    # pylint: disable=unused-argument
    def get_surcharges(self, basket, **kwargs):
        """
        For example::
            return (
                PercentageCharge(percentage=D("2.00")),
                FlatCharge(excl_tax=D("20.0"), incl_tax=D("20.0")),
            )

        Surcharges must implement the minimal API in ``oscar.apps.checkout.surcharges.BaseSurcharge``.
        Note that you can also make it a model if you want, just like shipping methods.
        """

        return ()

    def get_applicable_surcharges(self, basket, **kwargs):
        methods = [
            SurchargePrice(surcharge, surcharge.calculate(basket=basket, **kwargs))
            for surcharge in self.get_surcharges(basket=basket, **kwargs)
            if self.is_applicable(surcharge=surcharge, basket=basket, **kwargs)
        ]

        if methods:
            return SurchargeList(methods)
        else:
            return None

    # pylint: disable=unused-argument
    def is_applicable(self, surcharge, basket, **kwargs):
        """
        Checks if surcharge is applicable to certain conditions
        """
        return True



from django.conf import settings
from django.db.models.signals import post_save

from oscar.core.loading import get_class, get_model

AlertsDispatcher = get_class("customer.alerts.utils", "AlertsDispatcher")


# pylint: disable=unused-argument
def send_product_alerts(sender, instance, created, **kwargs):
    if kwargs.get("raw", False):
        return
    AlertsDispatcher().send_product_alert_email_for_user(instance.product)


if settings.OSCAR_EAGER_ALERTS:
    StockRecord = get_model("partner", "StockRecord")
    post_save.connect(send_product_alerts, sender=StockRecord)



from django.contrib import admin

from oscar.core.loading import get_model


class ProductRecordAdmin(admin.ModelAdmin):
    list_display = ("product", "num_views", "num_basket_additions", "num_purchases")


class UserProductViewAdmin(admin.ModelAdmin):
    list_display = ("user", "product", "date_created")


class UserRecordAdmin(admin.ModelAdmin):
    list_display = (
        "user",
        "num_product_views",
        "num_basket_additions",
        "num_orders",
        "total_spent",
        "date_last_order",
    )


admin.site.register(get_model("analytics", "productrecord"), ProductRecordAdmin)
admin.site.register(get_model("analytics", "userrecord"), UserRecordAdmin)
admin.site.register(get_model("analytics", "usersearch"))
admin.site.register(get_model("analytics", "userproductview"), UserProductViewAdmin)



from decimal import ROUND_UP
from decimal import Decimal as D

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

from oscar.core.loading import get_classes, get_model
from oscar.templatetags.currency_filters import currency

Condition = get_model("offer", "Condition")
range_anchor, unit_price = get_classes("offer.utils", ["range_anchor", "unit_price"])

__all__ = ["CountCondition", "CoverageCondition", "ValueCondition"]


class CountCondition(Condition):
    """
    An offer condition dependent on the NUMBER of matching items from the
    basket.
    """

    _description = _("Basket includes %(count)d item(s) from %(range)s")

    @property
    def name(self):
        return self._description % {
            "count": self.value,
            "range": str(self.range).lower(),
        }

    @property
    def description(self):
        return self._description % {
            "count": self.value,
            "range": range_anchor(self.range),
        }

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Count condition")
        verbose_name_plural = _("Count conditions")

    def is_satisfied(self, offer, basket):
        """
        Determines whether a given basket meets this condition
        """
        num_matches = 0
        for line in basket.all_lines():
            if self.can_apply_condition(line):
                num_matches += line.quantity_without_offer_discount(offer)
            if num_matches >= self.value:
                return True
        return False

    def _get_num_matches(self, basket, offer):
        if hasattr(self, "_num_matches"):
            return getattr(self, "_num_matches")
        num_matches = 0
        for line in basket.all_lines():
            if self.can_apply_condition(line):
                num_matches += line.quantity_available_for_offer(offer)
        # pylint: disable=W0201
        self._num_matches = num_matches
        return num_matches

    def is_partially_satisfied(self, offer, basket):
        num_matches = self._get_num_matches(basket, offer)
        return 0 < num_matches < self.value

    def get_upsell_message(self, offer, basket):
        num_matches = self._get_num_matches(basket, offer)
        delta = self.value - num_matches
        if delta > 0:
            return ngettext(
                "Buy %(delta)d more product from %(range)s",
                "Buy %(delta)d more products from %(range)s",
                int(delta),
            ) % {"delta": delta, "range": self.range}

    def consume_items(self, offer, basket, affected_lines):
        """
        Marks items within the basket lines as consumed so they
        can't be reused in other offers.

        :basket: The basket
        :affected_lines: The lines that have been affected by the discount.
                         This should be list of tuples (line, discount, qty)
        """
        # We need to count how many items have already been consumed as part of
        # applying the benefit, so we don't consume too many items.
        num_consumed = 0
        for line, __, quantity in affected_lines:
            num_consumed += quantity
        to_consume = max(0, self.value - num_consumed)
        if to_consume == 0:
            return

        for __, line in self.get_applicable_lines(
            offer, basket, most_expensive_first=True
        ):
            num_consumed = line.consume(to_consume, offer=offer)
            to_consume -= num_consumed
            if to_consume == 0:
                break


class CoverageCondition(Condition):
    """
    An offer condition dependent on the number of DISTINCT matching items from
    the basket.
    """

    _description = _("Basket includes %(count)d distinct item(s) from %(range)s")

    @property
    def name(self):
        return self._description % {
            "count": self.value,
            "range": str(self.range).lower(),
        }

    @property
    def description(self):
        return self._description % {
            "count": self.value,
            "range": range_anchor(self.range),
        }

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Coverage Condition")
        verbose_name_plural = _("Coverage Conditions")

    def is_satisfied(self, offer, basket):
        """
        Determines whether a given basket meets this condition
        """
        covered_ids = []
        for line in basket.all_lines():
            if not line.is_available_for_offer_discount(offer):
                continue
            product = line.product
            if self.can_apply_condition(line) and product.id not in covered_ids:
                covered_ids.append(product.id)
            if len(covered_ids) >= self.value:
                return True
        return False

    def _get_num_covered_products(self, basket, offer):
        covered_ids = set()
        for line in basket.all_lines():
            product = line.product
            if (
                self.can_apply_condition(line)
                and line.quantity_available_for_offer(offer) > 0
            ):
                covered_ids.add(product.id)
        return len(covered_ids)

    def get_upsell_message(self, offer, basket):
        delta = self.value - self._get_num_covered_products(basket, offer)
        if delta > 0:
            return ngettext(
                "Buy %(delta)d more product from %(range)s",
                "Buy %(delta)d more products from %(range)s",
                int(delta),
            ) % {"delta": delta, "range": self.range}

    def is_partially_satisfied(self, offer, basket):
        return 0 < self._get_num_covered_products(basket, offer) < self.value

    def consume_items(self, offer, basket, affected_lines):
        """
        Marks items within the basket lines as consumed so they
        can't be reused in other offers.
        """
        # Determine products that have already been consumed by applying the
        # benefit
        consumed_products = []
        for line, __, quantity in affected_lines:  # pylint: disable=W0612
            consumed_products.append(line.product)

        to_consume = max(0, self.value - len(consumed_products))
        if to_consume == 0:
            return

        for line in basket.all_lines():
            product = line.product
            if not self.can_apply_condition(line):
                continue
            if product in consumed_products:
                continue
            if not line.is_available_for_offer_discount(offer):
                continue
            # Only consume a quantity of 1 from each line
            line.consume(1, offer=offer)
            consumed_products.append(product)
            to_consume -= 1
            if to_consume == 0:
                break

    def get_value_of_satisfying_items(self, offer, basket):
        covered_ids = []
        value = D("0.00")
        for line in basket.all_lines():
            if self.can_apply_condition(line) and line.product.id not in covered_ids:
                covered_ids.append(line.product.id)
                value += unit_price(offer, line)
            if len(covered_ids) >= self.value:
                return value
        return value


class ValueCondition(Condition):
    """
    An offer condition dependent on the VALUE of matching items from the
    basket.
    """

    _description = _("Basket includes %(amount)s from %(range)s")

    @property
    def name(self):
        return self._description % {
            "amount": currency(self.value),
            "range": str(self.range).lower(),
        }

    @property
    def description(self):
        return self._description % {
            "amount": currency(self.value),
            "range": range_anchor(self.range),
        }

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Value condition")
        verbose_name_plural = _("Value conditions")

    def is_satisfied(self, offer, basket):
        """
        Determine whether a given basket meets this condition
        """
        value_of_matches = D("0.00")
        for line in basket.all_lines():
            if (
                self.can_apply_condition(line)
                and line.quantity_without_offer_discount(offer) > 0
            ):
                price = unit_price(offer, line)
                value_of_matches += price * int(
                    line.quantity_without_offer_discount(offer)
                )
            if value_of_matches >= self.value:
                return True
        return False

    def _get_value_of_matches(self, offer, basket):
        if hasattr(self, "_value_of_matches"):
            return getattr(self, "_value_of_matches")
        value_of_matches = D("0.00")
        for line in basket.all_lines():
            if self.can_apply_condition(line):
                price = unit_price(offer, line)
                value_of_matches += price * int(
                    line.quantity_available_for_offer(offer)
                )
        self._value_of_matches = value_of_matches  # pylint: disable=W0201
        return value_of_matches

    def is_partially_satisfied(self, offer, basket):
        value_of_matches = self._get_value_of_matches(offer, basket)
        return D("0.00") < value_of_matches < self.value

    def get_upsell_message(self, offer, basket):
        value_of_matches = self._get_value_of_matches(offer, basket)
        delta = self.value - value_of_matches
        if delta > 0:
            return _("Spend %(value)s more from %(range)s") % {
                "value": currency(delta, basket.currency),
                "range": self.range,
            }

    def consume_items(self, offer, basket, affected_lines):
        """
        Marks items within the basket lines as consumed so they
        can't be reused in other offers.

        We allow lines to be passed in as sometimes we want them sorted
        in a specific order.
        """
        # Determine value of items already consumed as part of discount
        value_consumed = D("0.00")
        for line, __, qty in affected_lines:
            price = unit_price(offer, line)
            value_consumed += price * qty

        to_consume = max(0, self.value - value_consumed)
        if to_consume == 0:
            return

        for price, line in self.get_applicable_lines(
            offer, basket, most_expensive_first=True
        ):
            quantity_to_consume = min(
                line.quantity_without_offer_discount(offer),
                (to_consume / price).quantize(D(1), ROUND_UP),
            )
            line.consume(quantity_to_consume, offer=offer)
            to_consume -= price * quantity_to_consume
            if to_consume <= 0:
                break



import datetime

from django import forms
from django.http import QueryDict
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy

from oscar.core.loading import get_class, get_model
from oscar.forms.mixins import PhoneNumberMixin
from oscar.forms.widgets import DatePickerInput

Order = get_model("order", "Order")
OrderNote = get_model("order", "OrderNote")
ShippingAddress = get_model("order", "ShippingAddress")
SourceType = get_model("payment", "SourceType")
AbstractAddressForm = get_class("address.forms", "AbstractAddressForm")


class OrderStatsForm(forms.Form):
    date_from = forms.DateField(
        required=False,
        label=pgettext_lazy("start date", "From"),
        widget=DatePickerInput,
    )
    date_to = forms.DateField(
        required=False, label=pgettext_lazy("end date", "To"), widget=DatePickerInput
    )

    _filters = _description = None

    def _determine_filter_metadata(self):
        self._filters = {}
        self._description = _("All orders")
        if self.errors:
            return

        date_from = self.cleaned_data["date_from"]
        date_to = self.cleaned_data["date_to"]
        if date_from and date_to:
            # We want to include end date so we adjust the date we use with the
            # 'range' function.
            self._filters = {
                "date_placed__range": [date_from, date_to + datetime.timedelta(days=1)]
            }
            self._description = _(
                "Orders placed between %(date_from)s and %(date_to)s"
            ) % {"date_from": date_from, "date_to": date_to}
        elif date_from and not date_to:
            self._filters = {"date_placed__gte": date_from}
            self._description = _("Orders placed since %s") % (date_from,)
        elif not date_from and date_to:
            self._filters = {"date_placed__lte": date_to}
            self._description = _("Orders placed until %s") % (date_to,)
        else:
            self._filters = {}
            self._description = _("All orders")

    def get_filters(self):
        if self._filters is None:
            self._determine_filter_metadata()
        return self._filters

    def get_filter_description(self):
        if self._description is None:
            self._determine_filter_metadata()
        return self._description


class OrderSearchForm(forms.Form):
    order_number = forms.CharField(required=False, label=_("Order number"))
    name = forms.CharField(required=False, label=_("Customer name"))
    product_title = forms.CharField(required=False, label=_("Product name"))
    upc = forms.CharField(required=False, label=_("UPC"))
    partner_sku = forms.CharField(required=False, label=_("Partner SKU"))

    status_choices = (("", "---------"),) + tuple(
        [(v, v) for v in Order.all_statuses()]
    )
    status = forms.ChoiceField(
        choices=status_choices, label=_("Status"), required=False
    )

    date_from = forms.DateField(
        required=False, label=_("Date from"), widget=DatePickerInput
    )
    date_to = forms.DateField(
        required=False, label=_("Date to"), widget=DatePickerInput
    )

    voucher = forms.CharField(required=False, label=_("Voucher code"))

    payment_method = forms.ChoiceField(
        label=_("Payment method"), required=False, choices=()
    )

    format_choices = (
        ("html", _("HTML")),
        ("csv", _("CSV")),
    )
    response_format = forms.ChoiceField(
        widget=forms.RadioSelect,
        required=False,
        choices=format_choices,
        initial="html",
        label=_("Get results as"),
    )

    def __init__(self, *args, **kwargs):
        # Ensure that 'response_format' is always set
        if "data" in kwargs:
            data = kwargs["data"]
            del kwargs["data"]
        elif len(args) > 0:
            data = args[0]
            args = args[1:]
        else:
            data = None

        if data:
            if data.get("response_format", None) not in self.format_choices:
                # Handle POST/GET dictionaries, which are immutable.
                if isinstance(data, QueryDict):
                    data = data.dict()
                data["response_format"] = "html"

        super().__init__(data, *args, **kwargs)
        self.fields["payment_method"].choices = self.payment_method_choices()

    def payment_method_choices(self):
        return (("", "---------"),) + tuple(
            [(src.code, src.name) for src in SourceType.objects.all()]
        )


class OrderNoteForm(forms.ModelForm):
    class Meta:
        model = OrderNote
        fields = ["message"]

    def __init__(self, order, user, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instance.order = order
        self.instance.user = user


class ShippingAddressForm(PhoneNumberMixin, AbstractAddressForm):
    class Meta:
        model = ShippingAddress
        fields = [
            "first_name",
            "last_name",
            "line1",
            "line2",
            "line3",
            "line4",
            "state",
            "postcode",
            "country",
            "phone_number",
            "notes",
        ]


class OrderStatusForm(forms.Form):
    new_status = forms.ChoiceField(label=_("New order status"), choices=())

    def __init__(self, order, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Set the choices
        choices = [(x, x) for x in order.available_statuses()]
        self.fields["new_status"].choices = choices

    @property
    def has_choices(self):
        return len(self.fields["new_status"].choices) > 0



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-04 05:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0005_auto_20170423_1217'),
    ]

    operations = [
        migrations.AlterField(
            model_name='conditionaloffer',
            name='end_datetime',
            field=models.DateTimeField(blank=True, help_text='Offers are active until the end date. Leave this empty if the offer has no expiry date.', null=True, verbose_name='End date'),
        ),
        migrations.AlterField(
            model_name='conditionaloffer',
            name='start_datetime',
            field=models.DateTimeField(blank=True, help_text='Offers are active from the start date. Leave this empty if the offer has no start date.', null=True, verbose_name='Start date'),
        ),
    ]



from collections import defaultdict

from django.db import models
from django.db.models import Exists, OuterRef, Prefetch, F, Q
from django.db.models.constants import LOOKUP_SEP
from treebeard.mp_tree import MP_NodeQuerySet

from oscar.core.loading import get_model


class AttributeFilter(dict):
    """
    Utility class used to implement the filter_by_attributes functionality.

    handles lookups, options and multivalue properties, check the tests for
    all features.
    """

    def __init__(self, filter_kwargs):
        super(AttributeFilter, self).__init__()

        for key, value in filter_kwargs.items():
            if LOOKUP_SEP in key:
                field_name, lookup = key.split(LOOKUP_SEP, 1)
                self[field_name] = (lookup, value)
            else:
                self[key] = (None, value)

    def field_names(self):
        return self.keys()

    def _selector(self, attribute_type):
        if attribute_type == "option" or attribute_type == "multi_option":
            return "attribute_values__value_%s__option" % attribute_type
        else:
            return "attribute_values__value_%s" % attribute_type

    def _select_value(self, types, lookup, value):
        _filter = models.Q()
        for _type in types:
            sel = self._selector(_type)
            if lookup is not None:
                sel = "%s%s%s" % (sel, LOOKUP_SEP, lookup)

            kwargs = dict()
            kwargs[sel] = value
            _filter |= models.Q(**kwargs)

        return _filter

    def fast_query(self, attribute_types, queryset):
        qs = queryset
        typedict = defaultdict(list)

        for code, attribute_type in attribute_types:
            typedict[code].append(attribute_type)

        for code, (lookup, value) in self.items():
            selected_values = self._select_value(typedict[code], lookup, value)
            if (
                not selected_values
            ):  # if no value clause can be formed, no result can be formed.
                return queryset.none()

            qs = qs.filter(
                selected_values,
                attribute_values__attribute__code=code,
            )

        return qs


class ProductQuerySet(models.query.QuerySet):
    def filter_by_attributes(self, **filter_kwargs):
        """
        Allows querying by attribute as if the attributes where fields on the
        product::

        >>> first_large_shirt = Product.objects.filter_by_attributes(size="Large").first()
        >>> first_large_shirt.attr.size
        <AttributeOption: Large>
        """
        attribute_filter = AttributeFilter(filter_kwargs)

        ProductAttribute = self.model.attributes.rel.model
        attribute_types = ProductAttribute.objects.values_list("code", "type").filter(
            code__in=attribute_filter.field_names()
        )

        return attribute_filter.fast_query(attribute_types, self)

    def base_queryset(self):
        """
        Applies select_related and prefetch_related for commonly related
        models to save on queries
        """
        Option = get_model("catalogue", "Option")
        product_class_options = Option.objects.filter(
            productclass=OuterRef("product_class")
        )
        product_options = Option.objects.filter(product=OuterRef("pk"))
        return (
            self.select_related("product_class")  # pylint:disable=E1102
            .annotate(
                has_product_class_options=Exists(product_class_options),
                has_product_options=Exists(product_options),
            )
            .prefetch_related(
                "children",
                "product_options",
                "product_class__options",
                "stockrecords",
                "images",
            )
        )

    def browsable(self):
        """
        Excludes non-canonical products and non-public products
        """
        return self.filter(parent=None, is_public=True)

    def public(self):
        """
        Excludes non-public products
        """
        return self.filter(is_public=True)

    def browsable_dashboard(self):
        """
        Products that should be browsable in the dashboard.

        Excludes non-canonical products, but includes non-public products.
        """
        return self.filter(parent=None)

    def prefetch_browsable_categories(self, queryset=None):
        """
        Prefetches browsable categories for each product in the queryset,
        including the parent's categories
        """
        if queryset is None:
            Category = get_model("catalogue", "Category")
            queryset = Category.objects.browsable()

        return self.prefetch_related(
            Prefetch(
                "categories",
                queryset=queryset,
                to_attr="_prefetched_browsable_categories",
            ),
            Prefetch(
                "parent__categories",
                queryset=queryset,
                to_attr="_prefetched_browsable_categories",
            ),
        )

    def prefetch_public_children(self, queryset=None):
        """
        Prefetches public children for each product in the queryset
        """
        if queryset is None:
            queryset = self.model.objects.public()

        return self.prefetch_related(
            Prefetch(
                "children",
                queryset=queryset,
                to_attr="_prefetched_public_children",
            )
        )

    def prefetch_attribute_values(self, include_parent_children_attributes=False):
        """
        This prefetches the attribute values for each product in the queryset.
        It also makes sure that for child products, the parent's attribute values are also
        prefetched (excluding the ones the child has).

        Args:
            include_parent_children_attributes (bool): If True, it will also prefetch the attributes for the
            parent's children. You should only set this to true if you're going to iterate over the
            parent's children's attribute values as well, otherwise you're doing useless queries.
        """
        ProductAttributeValue = get_model("catalogue", "ProductAttributeValue")
        AttributeOption = get_model("catalogue", "AttributeOption")
        ProductAttribute = get_model("catalogue", "ProductAttribute")

        # The base queryset for both self and parent attribute values.
        prefetch_queryset = (
            ProductAttributeValue.objects.all()
            .select_related("attribute", "value_option", "value_option__group")
            .prefetch_related(
                Prefetch(
                    "value_multi_option",
                    queryset=AttributeOption.objects.select_related("group"),
                )
            )
            .annotate(code=F("attribute__code"))
        )

        # pylint: disable=not-callable
        queryset = self.select_related(
            "product_class", "parent__product_class"
        ).prefetch_related(
            Prefetch(
                "attribute_values",
                queryset=prefetch_queryset,
                to_attr="_prefetched_attribute_values",
            ),
            Prefetch(
                "parent__attribute_values",
                queryset=prefetch_queryset,
                to_attr="_prefetched_parent_attribute_values",
            ),
            # The AttributesQuerysetCache retrieves the attributes for the product class, prefetch those too.
            Prefetch(
                "product_class__attributes",
                queryset=ProductAttribute.objects.all(),
            ),
            Prefetch(
                "parent__product_class__attributes",
                queryset=ProductAttribute.objects.all(),
            ),
        )

        if include_parent_children_attributes:
            queryset = queryset.prefetch_related(
                Prefetch(
                    "children__attribute_values",
                    queryset=prefetch_queryset,
                    to_attr="_prefetched_attribute_values",
                ),
                Prefetch(
                    "children__parent__attribute_values",
                    queryset=prefetch_queryset,
                    to_attr="_prefetched_parent_attribute_values",
                ),
            )

        return queryset


class CategoryQuerySet(MP_NodeQuerySet):
    def browsable(self):
        """
        Excludes non-public categories
        """
        return self.filter(is_public=True, ancestors_are_public=True)

    def for_menu(self):
        """
        Browsable categories that are not excluded for the menu
        """
        excluded_paths = list(
            self.browsable()
            .filter(exclude_from_menu=True)
            .values_list("path", flat=True)
        )
        qs = self.browsable().filter(exclude_from_menu=False)
        if excluded_paths:
            condition = Q()
            for path in excluded_paths:
                condition |= Q(path__startswith=path)
            qs = qs.exclude(condition)
        return qs



class FailedPreCondition(Exception):
    def __init__(self, url, message=None, messages=None):
        self.url = url
        if message:
            self.messages = [message]
        elif messages:
            self.messages = messages
        else:
            self.messages = []


class PassedSkipCondition(Exception):
    """
    To be raised when a skip condition has been passed and the current view
    should be skipped. The passed URL dictates where to.
    """

    def __init__(self, url):
        self.url = url



from django.db.models.expressions import Subquery

EXPAND_UPWARDS_CATEGORY_QUERY = """
(SELECT "CATALOGUE_CATEGORY_JOIN"."id" FROM "catalogue_category" AS "CATALOGUE_CATEGORY_BASE"
LEFT JOIN "catalogue_category" AS "CATALOGUE_CATEGORY_JOIN" ON (
    "CATALOGUE_CATEGORY_BASE"."path" LIKE "CATALOGUE_CATEGORY_JOIN"."path" || '%%%%'
    AND "CATALOGUE_CATEGORY_BASE"."depth" >= "CATALOGUE_CATEGORY_JOIN"."depth"
)
WHERE "CATALOGUE_CATEGORY_BASE"."id" IN (%(subquery)s))
"""


EXPAND_DOWNWARDS_CATEGORY_QUERY = """
(SELECT "CATALOGUE_CATEGORY_JOIN"."id" FROM "catalogue_category" AS "CATALOGUE_CATEGORY_BASE"
LEFT JOIN "catalogue_category" AS "CATALOGUE_CATEGORY_JOIN" ON (
    "CATALOGUE_CATEGORY_JOIN"."path" LIKE "CATALOGUE_CATEGORY_BASE"."path" || '%%%%'
    AND "CATALOGUE_CATEGORY_BASE"."depth" <= "CATALOGUE_CATEGORY_JOIN"."depth"
)
WHERE "CATALOGUE_CATEGORY_BASE"."id" IN (%(subquery)s))
"""


# pylint: disable=abstract-method
class ExpandUpwardsCategoryQueryset(Subquery):
    template = EXPAND_UPWARDS_CATEGORY_QUERY

    def as_sqlite(self, compiler, connection):
        return super().as_sql(compiler, connection, self.template[1:-1])


# pylint: disable=abstract-method
class ExpandDownwardsCategoryQueryset(Subquery):
    template = EXPAND_DOWNWARDS_CATEGORY_QUERY

    def as_sqlite(self, compiler, connection):
        return super().as_sql(compiler, connection, self.template[1:-1])



# Generated by Django 5.1.1 on 2024-11-05 14:16
from django.db import migrations
from django.db.models import Q
from django.contrib.auth import get_user_model


def assign_permissions_to_staff_users(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    Permission = apps.get_model("auth", "Permission")
    dashboard_permissions_group, _ = Group.objects.get_or_create(
        name="All Oscar Dashboard Permissions"
    )
    User = get_user_model()
    UserGroup = User.groups.through
    user_groups = []
    
    permission_codes = [
        # Catalogue
        "catalogue.add_product",
        "catalogue.change_product",
        "catalogue.view_product",
        "catalogue.delete_product",
        "catalogue.add_category",
        "catalogue.change_category",
        "catalogue.view_category",
        "catalogue.delete_category",
        "catalogue.add_productclass",
        "catalogue.change_productclass",
        "catalogue.view_productclass",
        "catalogue.delete_productclass",
        "catalogue.add_attributeoptiongroup",
        "catalogue.change_attributeoptiongroup",
        "catalogue.view_attributeoptiongroup",
        "catalogue.delete_attributeoptiongroup",
        "catalogue.add_option",
        "catalogue.change_option",
        "catalogue.view_option",
        "catalogue.delete_option",
        
        # Communications
        "communication.add_communicationeventtype",
        "communication.change_communicationeventtype",
        "communication.view_communicationeventtype",
        "communication.delete_communicationeventtype",
        
        # Partners
        "partner.add_stockalert",
        "partner.change_stockalert",
        "partner.view_stockalert",
        "partner.delete_stockalert",
        "partner.add_partner",
        "partner.change_partner",
        "partner.view_partner",
        "partner.delete_partner",
        
        # Offers
        "offer.add_conditionaloffer",
        "offer.change_conditionaloffer",
        "offer.view_conditionaloffer",
        "offer.delete_conditionaloffer",
        
        # Orders
        "order.add_order",
        "order.change_order",
        "order.view_order",
        "order.delete_order",
        
        # Flat Pages
        "flatpages.add_flatpage",
        "flatpages.change_flatpage",
        "flatpages.view_flatpage",
        "flatpages.delete_flatpage",
        
        # Ranges
        "offer.add_range",
        "offer.change_range",
        "offer.view_range",
        "offer.delete_range",
       
        # Analytics
        "analytics.add_userrecord",
        "analytics.change_userrecord",
        "analytics.view_userrecord",
        "analytics.delete_userrecord",
        
        # Reviews
        "reviews.add_productreview",
        "reviews.change_productreview",
        "reviews.view_productreview",
        "reviews.delete_productreview",
        
        # Shipping
        "add_weightbased",
        "change_weightbased",
        "delete_weightbased",
        "view_weightbased",

        # Users
        f"{User._meta.app_label}.add_user",
        f"{User._meta.app_label}.change_user",
        f"{User._meta.app_label}.view_user",
        f"{User._meta.app_label}.delete_user",
       
        # Vouchers
        "voucher.add_voucher",
        "voucher.change_voucher",
        "voucher.view_voucher",
        "voucher.delete_voucher",
    ]

    filters = Q()
    for code in permission_codes:
        app_label, __, codename = code.partition(".")
        filters |= Q(content_type__app_label=app_label, codename=codename)
    permissions = Permission.objects.filter(filters)
    dashboard_permissions_group.permissions.set(permissions)
    dashboard_permissions_group.save()

    for user_id in User.objects.filter(is_staff=True, is_superuser=False).values_list("id", flat=True):
        user_groups.append(
            UserGroup(user_id=user_id, group_id=dashboard_permissions_group.id)   
        )
    UserGroup.objects.bulk_create(user_groups)


class Migration(migrations.Migration):
    dependencies = [
        ("analytics", "0001_initial"),
        ("catalogue", "0001_initial"),
        ("communication", "0001_initial"),
        ("customer", "0007_auto_20200801_0817"),
        ("flatpages", "0001_initial"),
        ("offer", "0001_initial"),
        ("offer", "0001_initial"),
        ("order", "0001_initial"),
        ("partner", "0001_initial"),
        ("reviews", "0001_initial"),
        ("voucher", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(assign_permissions_to_staff_users, migrations.RunPython.noop)
    ]



from django.db import models


class CommunicationTypeManager(models.Manager):
    def get_and_render(self, code, context):
        """
        Return a dictionary of rendered messages, ready for sending.

        This method wraps around whether an instance of this event-type exists
        in the database.  If not, then an instance is created on the fly and
        used to generate the message contents.
        """
        try:
            commtype = self.get(code=code)
        except self.model.DoesNotExist:
            commtype = self.model(code=code)
        return commtype.get_messages(context)



from datetime import time

from django.http import HttpResponse
from django.template.defaultfilters import date
from django.utils.translation import gettext_lazy as _

from oscar.core import utils
from oscar.core.compat import UnicodeCSVWriter


class ReportGenerator(object):
    """
    Top-level class that needs to be subclassed to provide a
    report generator.
    """

    filename_template = "report-%s-to-%s.csv"
    content_type = "text/csv"
    code = ""
    description = "<insert report description>"
    date_range_field_name = None
    model_class = None
    queryset = None

    def __init__(self, **kwargs):
        self.start_date = kwargs.get("start_date")
        self.end_date = kwargs.get("end_date")

        formatter_name = "%s_formatter" % kwargs.get("formatter", "HTML")
        # pylint: disable=no-member
        self.formatter = self.formatters[formatter_name]()
        self.queryset = self.get_queryset()
        self.queryset = self.filter_with_date_range(self.queryset)

    def report_description(self):
        return _("%(report_filter)s between %(start_date)s and %(end_date)s") % {
            "report_filter": self.description,
            "start_date": date(self.start_date, "DATE_FORMAT"),
            "end_date": date(self.end_date, "DATE_FORMAT"),
        }

    def get_queryset(self):
        if self.queryset is not None:
            return self.queryset

        if not self.model_class:
            raise ValueError(
                "Please define a model_class property on your report generator class, "
                "or override the qet_queryset() method."
            )
        return self.model_class._default_manager.all()

    def generate(self):
        return self.formatter.generate_response(self.queryset)

    def filename(self):
        """
        Returns the filename for this report
        """
        return self.formatter.filename()

    def is_available_to(self, user):
        """
        Checks whether this report is available to this user
        """
        return user.is_staff

    def filter_with_date_range(self, queryset):
        """
        Filter results based that are within a (possibly open ended) daterange
        """
        # Nothing to do if we don't have a date field
        if not self.date_range_field_name:
            return queryset

        # After the start date
        if self.start_date:
            start_datetime = utils.datetime_combine(self.start_date, time.min)
            filter_kwargs = {
                "%s__gte" % self.date_range_field_name: start_datetime,
            }
            queryset = queryset.filter(**filter_kwargs)

        # Before the end of the end date
        if self.end_date:
            end_datetime = utils.datetime_combine(self.end_date, time.max)
            filter_kwargs = {
                "%s__lte" % self.date_range_field_name: end_datetime,
            }
            queryset = queryset.filter(**filter_kwargs)

        return queryset


class ReportFormatter(object):
    def format_datetime(self, dt):
        if not dt:
            return ""
        return utils.format_datetime(dt, "DATETIME_FORMAT")

    def format_date(self, d):
        if not d:
            return ""
        return utils.format_datetime(d, "DATE_FORMAT")

    def format_timedelta(self, td):
        return utils.format_timedelta(td)

    def filename(self):
        # pylint: disable=no-member
        return self.filename_template


class ReportCSVFormatter(ReportFormatter):
    def get_csv_writer(self, file_handle, **kwargs):
        return UnicodeCSVWriter(open_file=file_handle, **kwargs)

    def generate_response(self, objects, **kwargs):
        response = HttpResponse(content_type="text/csv")
        # pylint: disable=no-member
        response["Content-Disposition"] = "attachment; filename=%s" % self.filename(
            **kwargs
        )
        self.generate_csv(response, objects)
        return response


class ReportHTMLFormatter(ReportFormatter):
    def generate_response(self, objects, **kwargs):
        return objects



import keyword

from django.conf.urls.i18n import is_language_prefix_patterns_used
from django.core import validators
from django.core.exceptions import ValidationError
from django.http import Http404
from django.urls import get_urlconf, resolve
from django.utils.translation import get_language, get_language_from_path
from django.utils.translation import gettext_lazy as _
from django.utils.translation import override

from oscar.core.loading import get_model


class ExtendedURLValidator(validators.URLValidator):
    def __init__(self, *args, **kwargs):
        self.is_local_url = False
        super().__init__(*args, **kwargs)

    def __call__(self, value):
        try:
            super().__call__(value)
        except ValidationError:
            # The parent validator will raise an exception if the URL is not a
            # valid absolute URL so we test here to see if it is a local URL.
            if value:
                self.validate_local_url(value)
            else:
                raise

    def _validate_url(self, value):
        try:
            resolve(value)
        except Http404:
            # We load flatpages here as it causes a circular reference problem
            # sometimes.  FlatPages is None if not installed
            FlatPage = get_model("flatpages", "FlatPage")
            if FlatPage is not None:
                try:
                    FlatPage.objects.get(url=value)
                except FlatPage.DoesNotExist:
                    self.is_local_url = True
                else:
                    return
            raise ValidationError(_('The URL "%s" does not exist') % value)
        else:
            self.is_local_url = True

    def validate_local_url(self, value):
        value = self.clean_url(value)
        # If we have i18n pattern in the URLconf, by default it will be
        # resolved against default language by `LocaleRegexURLResolver`. In
        # this case, it won't resolve the path /de/catalogue/ when default
        # language code is "en-gb" and so that path validation won't pass,
        # which is incorrect. In order to work it around, we extract language
        # code from URL and override current locale within the locale prefix of
        # the URL.
        urlconf = get_urlconf()
        i18n_patterns_used, _ = is_language_prefix_patterns_used(urlconf)
        redefined_language = None
        if i18n_patterns_used:
            language = get_language_from_path(value)
            current_language = get_language()
            if language != current_language:
                redefined_language = language
        if redefined_language:
            with override(redefined_language):
                self._validate_url(value)
        else:
            self._validate_url(value)

    def clean_url(self, value):
        """
        Ensure url has a preceding slash and no query string
        """
        if value != "/":
            value = "/" + value.lstrip("/")
        q_index = value.find("?")
        if q_index > 0:
            value = value[:q_index]
        return value


class URLDoesNotExistValidator(ExtendedURLValidator):
    def __call__(self, value):
        """
        Validate that the URL does not already exist.

        The URL will be verified first and raises ``ValidationError`` when
        it is invalid. A valid URL is checked for existence and raises
        ``ValidationError`` if the URL already exists.

        This validation uses two calls to ExtendedURLValidator which can
        be slow. Be aware of this, when you use it.

        Returns ``None`` if URL is valid and does not exist.
        """
        try:
            self.validate_local_url(value)
        except ValidationError:
            # Page exists - that is what we want
            return
        raise ValidationError(_("Specified page already exists!"), code="invalid")


def non_whitespace(value):
    stripped = value.strip()
    if not stripped:
        raise ValidationError(_("This field is required"))
    return stripped


def non_python_keyword(value):
    if keyword.iskeyword(value):
        raise ValidationError(_("This field is invalid as its value is forbidden"))
    return value



from django.contrib import admin

from oscar.core.loading import get_model


class UserAddressAdmin(admin.ModelAdmin):
    readonly_fields = (
        "num_orders_as_billing_address",
        "num_orders_as_shipping_address",
    )


class CountryAdmin(admin.ModelAdmin):
    list_display = ["__str__", "display_order"]
    list_filter = ["is_shipping_country"]
    search_fields = ["name", "printable_name", "iso_3166_1_a2", "iso_3166_1_a3"]


admin.site.register(get_model("address", "useraddress"), UserAddressAdmin)
admin.site.register(get_model("address", "country"), CountryAdmin)
